package flowsim.cbdv;

import flowsim.core.*;
import flowsim.dv.*;

import flowsim.util.Log2;

/* Class Based Distance Vector Router */

import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;

public abstract class CBDVRouter extends DVRouterImpl
{
	static final boolean CBDV_DEBUG = false;

	static final int LINK_DELAY_NORM = 20000; // 20 ms.
	static final int MAX_BW = Integer.MAX_VALUE; // 2Gbit/s
	static final int MAX_BW_LOG = 31; // bits

	static final int RUN_TIME = 100000; // every 100 ms

			
	/* javac bug: inner class can't access inherited protected members! */
	/* ------ should be:
	final class RunEvent implements Event {
		public void trigger() {
			run();
			// reschedule
			s.schedule(RUN_TIME, new RunEvent());
		}
	}
	*/
	/* ------ is: */
	final void javacbug_run() { run(); }
	final void javacbug_schedule(long t, Event e) { s.schedule(t,e); }
	final class RunEvent implements Event {
		public void trigger() {
			javacbug_run();
			// reschedule
			javacbug_schedule(RUN_TIME, new RunEvent());
		}
	}
	/* ------ */

	public abstract double get_delay_factor();
	public abstract double get_hopcount_factor();
	public abstract double get_invbw_factor();

	protected int link_cost(int iface) {
		LinkManager lm = node.get_iface(iface);
		if(lm==null) return INFINITY;
		Link l = lm.get_link();
		if(l==null) return INFINITY;

		double cost;
		double invbw;

		/*********************************/
		invbw = get_invbw_factor() *
			(MAX_BW_LOG-Log2.log2(lm.get_rbw_notbe()))
			/ MAX_BW_LOG;
		cost =	get_delay_factor()*l.get_delay()/LINK_DELAY_NORM +
			get_hopcount_factor() + invbw;
			
		/*********************************/

		if(CBDV_DEBUG) s.debug(this,"link_cost("+iface+")="+cost+" (delay="+l.get_delay()+", rbw="+lm.get_rbw_notbe()+")");
		if(CBDV_DEBUG) s.debug(this,"invbw="+invbw);

		return (int) cost;
	}

	public CBDVRouter(Simulator s, Node n, int r) {
		super(s, n, r);

		s.schedule(RUN_TIME, new RunEvent());
	}
}
