package flowsim.cbdv;

import flowsim.core.*;

public class DSUnicastFlow extends UnicastFlowImpl
{
	static final int DS_SERVICE0  = 2; // traditional
	static final int DS_SERVICE1  = 3; // bw constrained
	static final int DS_SERVICE2  = 4;
	static final int DS_SERVICE3  = 5; // delay constrained
	static final int BW_THRESHOLD = 16000; // 16 kbit/s
	static final int DL_THRESHOLD = 60000; // 60 ms

	int max_delay;
	int total_delay;
	boolean is_be; 
	
	void to_ds() {
		if(is_be) {
			set_ds(FlowImpl.DS_BEST_EFFORT);
		}
		else {
			if(get_bw()>BW_THRESHOLD) {
				if(max_delay < DL_THRESHOLD) set_ds(DS_SERVICE2);
				else set_ds(DS_SERVICE1);
			}
			else {
				if(max_delay < DL_THRESHOLD) set_ds(DS_SERVICE3);
				else set_ds(DS_SERVICE0);
			}
		}
	}

	protected UnicastFlow mkchild(Node child_node) {
		DSUnicastFlow f = new DSUnicastFlow(s,child_node,get_to(),
			get_bw(),get_flowid(),get_router_port(),max_delay,is_be);
		f.set_parent(this);
		f.set_ds(get_ds());
		f.total_delay = total_delay;
		return f;
	}

	public void forward(LinkManager lm) {
		total_delay+=lm.get_link().get_delay();
		//System.out.println("max_delay="+max_delay+", total_delay="+total_delay);
		if(total_delay>max_delay) block();
		else super.forward(lm);
	}

	public DSUnicastFlow(Simulator s, Node from, Node to, int bw,
			int flow, int ralg, int max_delay, boolean is_be)
	{
		super(s, from, to, bw, flow, 0, ralg);
		this.max_delay = max_delay;
		this.is_be = is_be;
		to_ds();
	}
}
