package flowsim.cbdv;

import flowsim.core.*;
import flowsim.util.*;
import flowsim.dv.*;

import java.io.*;
import java.util.*;

final class CBDVFromChooser implements NodeChooser {
	final static int[] distrib = {
		0, // 0
		0, // 1
		10, // 2
		0, // 3
		0, // 4
		0, // 5
		0, // 6
		0, // 7
		0, // 8
		0, // 9
		0, // 10
		0  // 11
	};
	int distrib_total;
	Topology t;
	Random random;
	public Node get() {
		int n = (int)(random.nextDouble()*distrib_total);
		int next=0;
		for(int i=0; i<12; i++) {
			next+=distrib[i];
			if(n<next) return t.node(i);
		}
		return null;
	}
	CBDVFromChooser(Topology t, Random random) {
		this.t=t;
		this.random=random;
		for(int i=0; i<12; i++) {
			distrib_total+=distrib[i];
		}
	}
}

final class CBDVToChooser implements NodeChooser {
	final static int[] distrib = {
		0, // 0
		0, // 1
		0, // 2
		0, // 3
		0, // 4
		0, // 5
		0, // 6
		0, // 7
		0, // 8
		0, // 9
		10, // 10
		0  // 11
	};
	int distrib_total;
	Topology t;
	Random random;
	public Node get() {
		int n = (int)(random.nextDouble()*distrib_total);
		int next=0;
		for(int i=0; i<12; i++) {
			next+=distrib[i];
			if(n<next) return t.node(i);
		}
		return null;
	}
	CBDVToChooser(Topology t, Random random) {
		this.t=t;
		this.random=random;
		for(int i=0; i<12; i++) {
			distrib_total+=distrib[i];
		}
	}
}

final class CBDVFlowFactory implements UnicastFlowFactory {
	Simulator s;
	Distribution bw_dist;
	Distribution dl_dist;
	Random random;
	boolean dv;
	public UnicastFlow create(Node from, Node to) {
		int bw = (int) bw_dist.rand();
		int dl = (int) dl_dist.rand();
		boolean is_be = false;
		CBDVUnicastFlow f = new CBDVUnicastFlow(s, from, to, bw, 0, dl, bw, is_be);
		//System.out.println("New flow ("+from+"->"+to+
		//	", bw="+bw+", dl="+dl+", ds="+f.ds+",is_be="+is_be+")");
		if(dv) f.set_router_port(0);
		return f;
	}
	CBDVFlowFactory(Simulator s, boolean dv) {
		this.s = s;
		this.dv = dv;
		// from 1kbit to 32kbit
		bw_dist = new UniformDistribution(1000,32000,1235355);
		dl_dist = new UniformDistribution(20000,100000,5656);
		random  = new Random(10123);
	}
}

final public class TestCBDV
{
	static Simulator s;
	static Topology t;
	static LinkFactory lf;
	static LinkManagerFactory lmf;
	static boolean make_nam;
	static String namfile;

	static void init_dv() {
		init1();
		t.add_services(DVRouterImpl.factory(s),0);
		init2();
	}

	static void init_cbdv() {
		init1();
		t.add_services(CBDVRouter0.cbdv0_factory(s),0);
		t.add_services(CBDVRouter1.cbdv1_factory(s),1);
		t.add_services(CBDVRouter2.cbdv2_factory(s),2);
		t.add_services(CBDVRouter3.cbdv3_factory(s),3);
		init2();
	}

	static void init1() {
		s = new Simulator();
		t = new Topology(s,12,4,4,null);

		lf =  FixedDelayLink.randomdelay_factory(s,128000,10000,20000,0);
		lmf = SimpleLinkManager.factory(s);
	}

	static void init2() {
		// create links
		t.add_duplex_link(0,1,lf,lmf);
		t.add_duplex_link(0,4,lf,lmf);
		t.add_duplex_link(0,9,lf,lmf);
		t.add_duplex_link(1,2,lf,lmf);
		t.add_duplex_link(1,6,lf,lmf);
		t.add_duplex_link(2,3,lf,lmf);
		t.add_duplex_link(2,4,lf,lmf);
		t.add_duplex_link(3,7,lf,lmf);
		t.add_duplex_link(4,5,lf,lmf);
		t.add_duplex_link(4,8,lf,lmf);
		t.add_duplex_link(5,6,lf,lmf);
		t.add_duplex_link(6,7,lf,lmf);
		t.add_duplex_link(7,9,lf,lmf);
		t.add_duplex_link(7,11,lf,lmf);
		t.add_duplex_link(8,9,lf,lmf);
		t.add_duplex_link(9,10,lf,lmf);
		t.add_duplex_link(10,11,lf,lmf);

		// create nam log
		if(make_nam) t.make_nam_log(namfile);

		// run simulation for some time
		s.run(200000); // 200 ms
	}

	public static double test_dv(int load) {
		init_dv();
		return test(load, true);
	}

	public static double test_cbdv(int load) {
		init_cbdv();
		return test(load, false);
	}

	public static double test(int load, boolean dv) {
		TrafficGenerator tg;
		UnicastFlowFactory uff;

		int iat = 10000; // inter arrival time

		uff = new CBDVFlowFactory(s, dv);
		tg = new TrafficGenerator(
			s,
			t,
			new ExponentialDistribution(1.0/iat, 2341),
			new ExponentialDistribution(1.0/(iat*load), 9993),
			new CBDVFromChooser(t,new Random(10111)),
			new CBDVToChooser(t,new Random(10112)),
			uff,
			(int) 0.9*load
		);
		
		tg.start();

		run("["+load+"] pre-simulation: ", 10000000);
		tg.init_stats();

		run("["+load+"]     simulation: ", 50000000);

		System.err.println("load: "+load+" -> current_flows: "+
			tg.get_current_flows());

		//tg.stop();

		//s.run(10000000);

		return tg.get_block_rate();
	}

	static void run(String str, long time) {
		System.err.print(str);
		System.err.flush();
		show_progress(time);
		s.run(time);
		System.err.println();
	}

	static class ProgressEvent implements Event {
		public void trigger() { System.err.print("."); System.err.flush(); }
	}
	public static void show_progress(long time) {
		long dt = time/20;
		for(int i=1; i<=20; i++) {
			s.schedule(dt*i, new ProgressEvent());
		}
	}

	public static void main(String[] args) {
		boolean dv=false;
		int last_arg=args.length-1;
		if(last_arg>=0) {
			if(args[last_arg].equals("dv")) { dv=true; last_arg--; }
		}
		if(dv) System.err.println("DV Router");
		else System.err.println("CBDV Router");

		if(last_arg<0) {
			make_nam=false;
			for(int n=5; n<200; n+=5) {
				double res;
				if(dv) res=test_dv(n);
				else   res=test_cbdv(n);
				System.out.println(n+"\t"+res);
			}	
		}
		else {
			make_nam=true;
			int n = Integer.parseInt(args[0]);
			double res;
			if(dv) { 
				namfile = "testdv.nam";
				res=test_dv(n);
			}
			else {
				namfile = "testcbdv.nam";
				res=test_cbdv(n);
			}
			System.out.println(n+"\t"+res);
		}
	}
}
