package flowsim.core;

/** Flow with flow control such as TCP. */
public class FCBEFlowImpl extends UnicastFlowImpl implements FCBEFlow
{
	static final boolean DEBUG = false;
	static final boolean FC_ENDTOEND = true;

	int child_abw; // child constraint on desired_bw
	int parent_child_abw;

	class SetParentBwEvent implements Event {
		int cbw;
		public void trigger() {
			FCBEFlow fcparent = (FCBEFlow) parent;
			fcparent.set_child_available_bw(cbw);
		}
		SetParentBwEvent(int cbw) { this.cbw = cbw; }
	}
	
	class SetChildBwEvent implements Event {
		int pbw;
		public void trigger() { child.set_parent_bw(pbw); }
		SetChildBwEvent(int pbw) { this.pbw = pbw; }
	}
	
	protected UnicastFlow mkchild(Node child_node) {
		FCBEFlow f = new FCBEFlowImpl(s,child_node,to,flow_id,ds,router_port,this,bw);
		return f;
	}

	public void set_child_available_bw(int child_abw) {
		int old_bw = this.child_abw;
		if(this.child_abw==child_abw) return;

		if(DEBUG) s.debug(this,"set_child_available_bw: "+
			"child_abw="+child_abw+
			", desired_bw="+desired_bw);

		this.child_abw = child_abw;

		tell_parent_child_abw();

		adjust_desired_bw();
	}

	void tell_parent_child_abw() {
		int abw = available_bw;
		if(abw>child_abw) abw = child_abw;

		if(parent_child_abw == abw) return;
		
		if(parent!=null) {
			if(parent.get_iface()!=null) {
				s.schedule(parent.get_iface().get_link().get_delay(),
					new SetParentBwEvent(abw));
			}
			else {
				FCBEFlow fcchild = (FCBEFlow) child;
				fcchild.set_child_available_bw(abw);
			}
		}

		parent_child_abw = abw;
	}
	
	public void set_parent_bw(int parent_bw) {
		if(DEBUG) s.debug(this,"set_parent_bw: parent_bw="+parent_bw+", desired_bw="+desired_bw);

		if(this.parent_bw==parent_bw) return;
		this.parent_bw = parent_bw;
		adjust_desired_bw();
	}

	public void set_bw(int abw, int bw) {
		int old_bw = desired_bw;
		int old_abw = available_bw;

		super.set_bw(abw, bw);

		if(old_bw == bw && old_abw == abw) return;

		tell_parent_child_abw();
	}

	void adjust_desired_bw() {
		if(DEBUG) s.debug(this, "adjust_desired_bw: "+
			"child_abw="+child_abw+
			", parent_bw="+parent_bw+
			", desired_bw="+desired_bw+")");

		int new_desired_bw = parent_bw;
		if(!FC_ENDTOEND || parent==null) {
			if(new_desired_bw>child_abw) {
				new_desired_bw = child_abw;
			}
		}

		if(new_desired_bw!=desired_bw) {
			set_desired_bw(new_desired_bw);
			if(DEBUG) s.debug(this,"new desired_bw is "+new_desired_bw);
		}
	}

	FCBEFlowImpl(Simulator s, Node from, Node to, int flow,
		int ds, int rport, UnicastFlow parent, int parent_bw)
	{
		super(s,from,to,parent_bw,flow,ds,rport,parent);
		this.parent_bw = parent_bw;
		this.parent_child_abw = Integer.MAX_VALUE;
		this.child_abw = Integer.MAX_VALUE;
	}

	public FCBEFlowImpl(Simulator s, Node from, Node to, int flow,
		int ds, int rport)
	{
		super(s,from,to,Integer.MAX_VALUE,flow,ds,rport,null);
		this.parent_bw = Integer.MAX_VALUE;
		this.parent_child_abw = Integer.MAX_VALUE;
		this.child_abw = Integer.MAX_VALUE;
	}
}
