package flowsim.core;

import com.sun.java.util.collections.Collection;
import com.sun.java.util.collections.Iterator;
import com.sun.java.util.collections.LinkedList;

public class FlowImpl implements Flow
{
	Node from;
	Service from_service;
	int flow_id;
	int ds; // old 'TOS', "differentiated services"
	int bw, desired_bw, available_bw;
	boolean running=false;
	boolean blocked=false;
	Collection monitors;

	public static final int DS_UNKNOWN      = 0;
	public static final int DS_BEST_EFFORT  = 1;

	public final void attach_monitor(FlowMonitor monitor) {
		monitors.add(monitor);
	}

	public final void detach_monitor(FlowMonitor monitor) {
		monitors.remove(monitor);
	}

	public final Node get_from() {
		return from;
	}

	public final Service get_from_service() {
		return from_service;
	}

	public final void set_from_service(Service service) {
		from_service = service;
	}

	public final int get_bw() {
		return bw;
	}

	// Total BW used on every link.
	public int get_total_bw() {
		return bw;
	}

	public final int get_flowid() {
		return flow_id;
	}

	public final void set_flowid(int flowid) {
		this.flow_id = flowid;
	}

	public final int get_ds() {
		return ds;
	}

	public void set_ds(int ds) {
		this.ds = ds;
	}

	private void _set_bw(int bw) {
		if(this.bw==bw) return;

		int oldbw = this.bw;
		this.bw = bw;
		if(bw<0) {
			System.err.println("FlowImpl: set_bw("+bw+")");
			throw new RuntimeException();
		}

		// monitors
		Iterator i = monitors.iterator();
		while(i.hasNext()) {
			FlowMonitor m = (FlowMonitor) i.next();
			m.flow_changed_bw(this,oldbw);
		}
	}

	private void _set_abw(int available_bw) {
		if(this.available_bw==available_bw) return;

		int oldbw = this.desired_bw;
		this.available_bw = available_bw;

		// monitors
		Iterator i = monitors.iterator();
		while(i.hasNext()) {
			FlowMonitor m = (FlowMonitor) i.next();
			m.flow_changed_available_bw(this,oldbw);
		}
	}

	public void set_bw(int abw, int bw) {
		_set_abw(abw);
		_set_bw(bw);
	}

	public int get_desired_bw() {
		return desired_bw;
	}

	public void set_desired_bw(int desired_bw) {
		if(this.desired_bw==desired_bw) return;

		int oldbw = this.desired_bw;
		this.desired_bw = desired_bw;

		// monitors
		Iterator i = monitors.iterator();
		while(i.hasNext()) {
			FlowMonitor m = (FlowMonitor) i.next();
			m.flow_changed_desired_bw(this,oldbw);
		}
	}

	public int get_available_bw() {
		return available_bw;
	}

	public void start() {
		if(running==true) return;
		running=true;
		blocked=false;

		// monitors
		Iterator i = monitors.iterator();
		while(i.hasNext()) {
			FlowMonitor m = (FlowMonitor) i.next();
			m.flow_started(this);
		}
	}

	/* Called from the user the stop the flow. */
	public void stop() {
		if(running==false) return;
		running=false;

		// monitors
		Iterator i = monitors.iterator();
		while(i.hasNext()) {
			FlowMonitor m = (FlowMonitor) i.next();
			m.flow_stopped(this);
		}
	}

	/* Called when a flow is blocked. */
	public void block() {
		if(blocked==true || running==false) return;

		blocked=true;
		running=false;

		// monitors
		Iterator i = monitors.iterator();
		while(i.hasNext()) {
			FlowMonitor m = (FlowMonitor) i.next();
			m.flow_blocked(this);
		}
	}

	public boolean is_running() {
		return running;
	}

	public boolean is_blocked() {
		return blocked;
	}

	/* number from 0 to 1 */
	public double utility() {
		if(blocked) return 0;
		else return 1;
	}

	/* This function is called usually by a Router when a flow is
	 * forwarded through a link.
	 */
	public void forward(LinkManager lm) {
	}

	public FlowImpl(Node from, int bw, int flow, int ds) {
		this.from         = from;
		this.bw           = 0;
		this.desired_bw   = bw;
		this.available_bw = 0;
		this.flow_id      = flow;
		this.ds           = ds;
		this.from_service = null;
		this.monitors     = new LinkedList();
	}

	public FlowImpl(Service from, int bw, int flow, int ds) {
		this(from.get_node(),bw,flow,ds);
		this.from_service = from;
	}
}
