package flowsim.core;

// warning: caching of object (to reduce gc overhead) can be dangerous if an
// object is used after it's 'release'.
// disable it (USE_CACHE) if you think you can have such a problem.
final class LLEventQueueCache
{
	EventQueueElement free;
	EventQueueElement get(final long t, final Event e) {
		if(free == null) {
			return new EventQueueElement(t, e);
		}
		else {
			EventQueueElement el = free;
			free = el.next;
			// if(free!=null) free.prev=null;
			el.time=t;
			el.event=e;
			return el;
		}
	}

	void release(EventQueueElement e) {
		e.next = free;
		free = e;
	}
}

final class LLEventQueue implements EventQueue
{
	static final boolean USE_CACHE = true;

	final LLEventQueueCache cache;
	EventQueueElement first;
	int size;

	public int size() {
		return size;
	}

	private void put(final EventQueueElement e) {
		final long etime = e.time;

		size++;

		EventQueueElement i = null;
		EventQueueElement n = first;

		while(n!=null) {
			if(n.time>etime) break;
			i = n;
			n = n.next;
		}

		e.prev = i; e.next = n;
		if(i!=null) i.next = e;
		else        first  = e;
		if(n!=null) n.prev = e;
	}

	public EventQueueElement put(final long time, final Event event) {
		EventQueueElement e;
		if(USE_CACHE) {
			e = cache.get(time,event);
		}
		else {
			e = new EventQueueElement(time, event);
		}

		put(e);
		return e;
	}

	public EventQueueElement get() {
		size--;
	
		final EventQueueElement e = first;

		first = first.next;
		if(first!=null) first.prev = null;

		if(USE_CACHE) cache.release(e);

		return e;
	}

	public long first_time() {
		if(first!=null) {
			return first.time;
		}
		else {
			return Long.MAX_VALUE;
		}
	}

	public LLEventQueue() {
		if(USE_CACHE) {
			cache = new LLEventQueueCache();
		}
		else {
			cache = null;
		}
	}
}
