package flowsim.core;

import java.io.PrintWriter;

final class LinkBandwidthLogger implements LinkMonitor
{
	Simulator s;
	Link l;
	PrintWriter pw;
	int old_rbw;
	long old_time;
	LinkMonitor monitor; // we cascade them...
	long bw;

	public void add_flow(int fbw) {
		bw += fbw;
		log();
		monitor.add_flow(fbw);
	}

	public void remove_flow(int fbw) {
		bw -= fbw;
		log();
		monitor.remove_flow(fbw);
	}

	void log() {
		if(pw != null) {
			if(old_time != s.now()) {
				pw.println(old_time+"\t"+(l.get_bw()-old_rbw));
				old_time = s.now();
			}
			old_rbw  = l.get_rbw();
		}
	}

	public void flush() {
		log();

		if(pw != null) pw.println(old_time+"\t"+(l.get_bw()-old_rbw));

		old_time = s.now();
		old_rbw  = l.get_rbw();

		if(monitor!=null) monitor.flush();
	}

	public final void attach_monitor(LinkMonitor lmo) {
		monitor = lmo;
	}

	LinkBandwidthLogger(Simulator s, Link l, PrintWriter pw) {
		this.s  = s;
		this.l  = l;
		this.pw = pw;

		l.attach_monitor(this);
	}
}
