package flowsim.core;

import java.io.PrintWriter;

public abstract class LinkImpl implements Link
{
	Simulator s;
	Node from;
	Node to;
	int bw;	 /* bandwidth in bit/s */
	int rbw; /* residual bw */
	LinkMonitor monitor; // sorry: only one...
	//static long refused=0;

	public final void attach_monitor(LinkMonitor lmo) {
		monitor = lmo;
	}

	public final LinkMonitor get_monitor() {
		return monitor;
	}

	public final Node get_from() { return from; }

	public final Node get_to()   { return to; }

	public final int get_bw() { return bw; }

	public final int get_rbw() { return rbw; }

	public abstract int get_delay(); /* in microseconds (us) */

	public int get_min_delay() { return get_delay(); }
	public int get_max_delay() { return get_delay(); }

	public boolean add_flow(int fbw) {
		// System.out.println(this+"\t: bw="+bw+", rbw="+rbw);
		if(fbw<=rbw) {
			rbw -= fbw;
			if(monitor!=null) monitor.add_flow(fbw);
			return true;
		}
		else {
			//refused++;
			//System.out.println("refused="+refused);
			return false;
		}
	}

	public void remove_flow(int fbw) {
		//System.out.println(this+"\t: bw="+bw+", rbw="+rbw+", fbw="+fbw);
		int old_rbw = rbw;
		rbw += fbw;
		if(rbw>bw) {
			s.error(this,"Link: Wanted to remove more than available bandwidth!");
			rbw=bw; // shouldn't happen...
		}
		if(monitor!=null) monitor.remove_flow(rbw-old_rbw);
	}

	public final String toString() {
		return "Link("+from+"-"+to+")";
	}

	public LinkImpl(Simulator s, Node from, Node to, int bw) {
		this.s    = s;
		this.from = from;
		this.to   = to;
		this.bw = this.rbw = bw;
		this.monitor = null;
	}
};

