package flowsim.core;

import java.util.Enumeration;
import com.sun.java.util.collections.Collection;
import com.sun.java.util.collections.Iterator;
import com.sun.java.util.collections.LinkedList;

public abstract class LinkManagerImpl implements LinkManager
{
	final Link link;
	Collection monitors;

	public void attach_monitor(LinkManagerMonitor lmm) {
		monitors.add(lmm);
	}

	public void detach_monitor(LinkManagerMonitor lmm) {
		monitors.remove(lmm);
	}

	public Iterator flows() { return null; }

	/** Get node on the other side of the link */
	public final Node get_peer() {
		return link.get_to();
	}

	/** Get managed link */
	public final Link get_link() {
		return link;
	}

	/** Residual bandwith without counting best-effort */
	public int get_rbw_notbe() {
		return link.get_rbw();
	}

	/** Add new flow.
	  * @return Delay in micro-seconds
	  */
	public long add(Flow flow) {
		Iterator i = monitors.iterator();
		while(i.hasNext()) {
			LinkManagerMonitor lmm = (LinkManagerMonitor) i.next();
			lmm.flow_added(this, flow);
		}
		return get_link().get_delay();
	}

	/**
	  * Called when the bandwidth of a flow is changed (usually when it is
	  * made smaller by other linkmanagers
	  */
	public void bw_changed(Flow flow) {
		Iterator i = monitors.iterator();
		while(i.hasNext()) {
			LinkManagerMonitor lmm = (LinkManagerMonitor) i.next();
			lmm.flow_changed(this, flow);
		}
	}

	/** Remove flow */
	public void remove(Flow flow) {
		Iterator i = monitors.iterator();
		while(i.hasNext()) {
			LinkManagerMonitor lmm = (LinkManagerMonitor) i.next();
			lmm.flow_removed(this, flow);
		}
	}

	public void print_debug_info() {}

	public LinkManagerImpl(Link link) {
		this.link = link;
		this.monitors = new LinkedList();
	}
}
