package flowsim.core;

import java.io.PrintWriter;
import com.sun.java.util.collections.Collection;
import com.sun.java.util.collections.Iterator;
import com.sun.java.util.collections.LinkedList;
import com.sun.java.util.collections.Map;
import com.sun.java.util.collections.HashMap;

public final class LinkManagerNamLogger implements LinkManagerMonitor
{
	static final long DEFAULT_CYCLE_TIME=10000;
	final long cycle_time;
	final Simulator s;
	final Node node;
	final LinkManager lm;
	int link_bw;
	Iterator flows_cycle;
	final PrintWriter namfile;
	final boolean flowid_aggregates;

	class FlowData {
		final int color;
		int bw;
		FlowData(Flow f) {
			bw    = f.get_bw();
			color = f.get_flowid();
		}
	}

	class ReceiveEvent implements Event {
		FlowData f; int size;
		public void trigger() { receive(f,size); }
		ReceiveEvent(FlowData f, int size) { this.f=f; this.size=size; }
	}
	
	class SendNextEvent implements Event {
		public void trigger() { send_next(); }
	}
	
	class StartCycleEvent implements Event {
		public void trigger() { start_cycle(); }
	}
	
	public void flow_added(LinkManager lm, Flow f) {
		/*
		// Don't simulate packets for Packet-Flows!
		if(f instanceof Packet) {
			send_packet(
		}
		*/
	}

	public void flow_removed(LinkManager lm, Flow f) {}

	public void flow_changed(LinkManager lm, Flow f) {}


	Iterator get_flows() {
		LinkedList flows = new LinkedList();
		Iterator it = lm.flows();
		int total_bw=0;
		while(it.hasNext()) {
			Flow f = (Flow) it.next();
			flows.add(new FlowData(f));
			total_bw+=f.get_bw();
		}
		if(total_bw>lm.get_link().get_bw()) {
			s.error(this,"total_bw>lm.get_link().get_bw()");
			return null;
		}
		return flows.iterator();
	}

	Iterator get_flows_aggr() {
		Map flows = new HashMap();
		Iterator it = lm.flows();
		
		int total_bw=0;
		while(it.hasNext()) {
			Flow f = (Flow) it.next();
			Integer fid = new Integer(f.get_flowid());
			FlowData fd = (FlowData) flows.get(fid);
			if(fd==null) {
				fd = new FlowData(f);
				flows.put(fid, fd);
			}
			else {
				fd.bw += f.get_bw();
			}
			total_bw+=f.get_bw();
		}
		if(total_bw>lm.get_link().get_bw()) {
			s.error(this,"total_bw>lm.get_link().get_bw()");
			return null;
		}
		return flows.values().iterator();
	}

	void start_cycle() {
		// s.debug(this,"Start Cycle!");
		// schedule next cycle
		s.schedule(cycle_time,new StartCycleEvent());

		if(!flowid_aggregates) flows_cycle=get_flows();
		else                   flows_cycle=get_flows_aggr();

		// send packets
		if(flows_cycle!=null) send_next();
	}

	void send_next() {
		if(flows_cycle.hasNext()) {
			//s.debug(this,"hasNext...");
			FlowData f = (FlowData) flows_cycle.next();
			long t = (cycle_time-1)*f.bw/link_bw;
			int size = (int) (t*link_bw/8000000);
			namfile.println("h -t "+(double)s.now()/1000000+" -e "+size+
					" -s "+node+" -d "+lm.get_peer()+
					" -c "+f.color+" -i 0 -a "+f.color);
			// schedule stop
			s.schedule(t,new ReceiveEvent(f,size));
			// schedule next
			s.schedule(t,new SendNextEvent());
		}
		/*
		else {
			s.debug(this,"no more");
		}
		*/
	}

	void receive(FlowData f, int size) {
		namfile.println("r -t "+(double)s.now()/1000000+" -e "+size+
				" -s "+node+" -d "+lm.get_peer()+
				" -c "+f.color+" -i 0 -a "+f.color);
	}

	LinkManagerNamLogger(Simulator s, Node node, LinkManager lm,
			PrintWriter namfile, long cycle_time,
			boolean flowid_aggregates)
	{
		this.s=s;
		this.node=node;
		this.lm=lm;
		this.link_bw = lm.get_link().get_bw();
		this.namfile=namfile;
		this.cycle_time = cycle_time;
		this.flowid_aggregates = flowid_aggregates;

		start_cycle();
	}

	LinkManagerNamLogger(Simulator s, Node node, LinkManager lm, PrintWriter namfile)
	{
		this(s,node,lm,namfile,DEFAULT_CYCLE_TIME,false);
	}
}
