package flowsim.core;

public class NextHopPacket implements Packet, DataTransferMonitor
{
	final Simulator s;
	final LinkManager iface;
	final Message msg;
	final UnicastFlow uf;
	final DataTransfer dt;

	/*****************************/
	/***** PACKET INTERFACE ******/
	/*****************************/

	public Message get_msg() {
		return msg;
	}

	public int get_remaining_bits() {
		return dt.get_remaining_bits();
	}

	public void start() {
		dt.start();
	}

	/*******************************************/
	/***** DATATRANSFER_MONITOR INTERFACE ******/
	/*******************************************/

	public void datatransfer_arrived(DataTransfer dt) {
		msg.arrived(iface.get_peer());
	}

	public void datatransfer_started(DataTransfer dr) {}
	public void datatransfer_changed_bw(DataTransfer dr) {}
	public void datatransfer_transmitted(DataTransfer dt) {}
	public void datatransfer_blocked(DataTransfer dr) {}

	/************************/
	/***** CONSTRUCTOR ******/
	/************************/
	
	public NextHopPacket(Simulator s, Node from, LinkManager iface,
		Message msg, int flow_id)
	{
		this.s     = s;
		this.msg   = msg;
		this.iface = iface;
		this.uf    = new NextHopFlow(from,iface, Integer.MAX_VALUE,
					flow_id, FlowImpl.DS_BEST_EFFORT);
		this.dt    = new DataTransferImpl(s,-1,-1,msg.size()*8, uf);
		this.dt.attach_monitor(this);
	}
}
