package flowsim.core;

import com.sun.java.util.collections.Collection;
import com.sun.java.util.collections.Iterator;
import com.sun.java.util.collections.LinkedList;

public class NodeImpl implements Node
{
	final boolean DEBUG = false;
	Simulator s;
	LinkManager[] ifaces;
	int ifaces_count;
	Service[] services;
	String name;
	final int id;
	static int next_id=1;
	Collection monitors;

	public final int get_id() {
		return id;
	}

	public final String get_name() {
		return name;
	}

	public final String toString() {
		return get_name();
	}

	public final int get_ifaces_count() {
		return ifaces_count;
	}

	public final int get_max_ifaces() {
		return (int) ifaces.length;
	}

	public final LinkManager get_iface(int i) {
		if(ifaces[i]==null) return null;
		else return ifaces[i];
	}

	public final int get_iface(Node n) {
		for(int i=0; i<ifaces.length; i++) {
			if(ifaces[i]==null) continue;
			if(ifaces[i].get_peer()==n) return i;
		}
		return -1;
	}

	public final Link get_link(int n) {
		if(ifaces[n]==null) return null;
		else return ifaces[n].get_link();
	}

	public final Node get_peer(int n) {
		if(ifaces[n]==null) return null;
		else return ifaces[n].get_peer();
	}

	public void set_iface(int n, LinkManager f) {
		boolean changed=true;
		if(ifaces[n]==f) changed=false;

		if(ifaces[n]!=null && f==null) {
			debug("removing link to node "+ifaces[n].get_peer().get_name()+".");
		}

		ifaces[n] = f;
		if(changed) iface_changed(n);
	}

	public int add_iface(LinkManager f) {
		debug("add_iface()");
		for(int i=0; i<ifaces.length; i++) {
			if(ifaces[i]==null) {
				ifaces[i] = f;
				ifaces_count++;
				iface_changed(i);
				return i;
			}
		}

		return -1;
	}

	public final Service get_service(int n) {
		return services[n];
	}

	public void set_service(Service r, int route) {
		debug("setting service.");
		services[route] = r;
	}

	void iface_changed(int iface) {
		debug("neighbour changed (iface nr. "+iface+").");
		Iterator i = monitors.iterator();
		while(i.hasNext()) {
			NodeMonitor nm = (NodeMonitor) i.next();
			nm.iface_changed((Node) this, iface);
		}
	}

	final void debug(String s) {
		if(DEBUG) System.out.println("Node("+name+"):\t"+s);
	}

	public boolean equals(Object obj) {
		return id==((Node) obj).get_id();
	}

	public void attach_monitor(NodeMonitor nm) {
		monitors.add(nm);
	}

	public void remove_monitor(NodeMonitor nm) {
		monitors.remove(nm);
	}

	public NodeImpl(Simulator s, String name, int ifaces_nr, int services_nr) {
		this.s    = s;
		this.name = name;
		this.id   = s.next_node_id++;
		ifaces  = new LinkManager[ifaces_nr];
		services = new Service[services_nr];
		monitors = new LinkedList();
	}
}
