package flowsim.core;

/**
 * Simulation of a single packet with a flow. The packet is simulated by
 * keeping the flow running for the amount of time needed to transport all the
 * data. The data contained in the packet is represented by a Message object,
 * which determines the it's size and what should be done when the packet
 * arrives on a node. To define new messages sub-class Message, not Packet!
 * @see Message
 */
public interface Packet
{
	/**
	 * Get message contained in this packet.
	 */
	Message get_msg();

	/**
	 * Get bits which remain to be transmitted.
	 */
	int get_remaining_bits();

	/**
	 * Start transmission.
	 */
	void start();
}
