package flowsim.core;

/**
 * Interface implemented by flows which can be routed by Routers. The
 * router_port property is used to get the right router on the nodes (via
 * node.get_service(router_port))
 * 
 * @see Router
 * @see Node#get_service
 */
public interface RoutedFlow extends Flow
{
	/**
	 *  Get router port of the routers which should process this flow.
	 */
	int get_router_port();

	/**
	 *  Set router port of the routers which should process this flow.
	 */
	void set_router_port(int port);

	/**
	 * Forward this flow on a interface. This function is normally called
	 * by a Router when it has decided on which interface this flow should
	 * go next.
	 * @param lm the interface (in form of a LinkManager) on which the
	 * flow should be forwarded.
	 * @see Router
	 */
	void forward(LinkManager lm);
}
