package flowsim.core;

abstract public class RoutedFlowImpl extends FlowImpl implements RoutedFlow
{
	static final boolean DEBUG_RF=false;
	protected Simulator s;
	int router_port;

	class NextEvent implements Event {
		Node n;
		public void trigger() { next_node(n); }
		NextEvent(Node n) { this.n = n; }
	}

	public final int get_router_port() {
		return router_port;
	}

	public final void set_router_port(int port) {
		this.router_port=router_port;
	}

	// Called when the forwarded flow arrives on the peer node
	abstract void next_node(Node next);

	public void forward(LinkManager lm) {
		if(DEBUG_RF) s.debug(this,"forwarding to link-manager (->"+lm.get_peer()+")");
		long delay = lm.add(this);
		if(delay>=0) {
			Event event = new NextEvent(lm.get_peer());
			s.schedule(delay,event);
		}
	}

	// Called to route this flow
	public void route() {
		if(DEBUG_RF) s.debug(this,"Route!");
		Router r = (Router) from.get_service(router_port);
		r.route(this);
	}

	public void start() {
		super.start();
		if(DEBUG_RF) s.debug(this,"Start!");
		route();
	}

	void debug(String str) {}

	public RoutedFlowImpl(Simulator s, Node from, int bw,
		int flow, int type, int router_port)
	{
		super(from,bw,flow,type);
		this.router_port = router_port;
		this.s = s;
	}
}
