package flowsim.core;

public final class Scheduler
{
	public static final long  TIME_INFINITY = Long.MAX_VALUE;
	static final boolean DEBUG = false;
	static final boolean TRACE = false;
	long time;
	final EventQueue events;
	final Simulator s;

	public long now() {
		return time;
	}

	public void run(final long how_long)
	{
		final long next_time = time + how_long;

		if(DEBUG) s.debug(this,"run (next_time = "+next_time+")");

		while(events.first_time() <= next_time) {
			final EventQueueElement e = events.get();
			if(time>e.time) {
				s.error(this, "Going backward in the past!");
				throw new RuntimeException();
			}
			time = e.time;
			if(TRACE) s.debug(this,"EVENT: "+e.event);

			final Event ev = e.event;
			if(ev!=null) ev.trigger();
		}

		time = next_time;
	}

	EventQueueElement schedule_absolute(final long when, final Event e)
	{
		if(DEBUG) s.debug(this,"adding "+e+" at "+when);
		if(when<time) {
			s.error(this,"Event in the past scheduled! (when="+when+")");
			throw new RuntimeException();
		}
		return events.put(when,e);
	}

	public EventQueueElement schedule(final long when, final Event e)
	{
		long t=time+when;
		if(t<0) {
			// wrapped => INFINITY
			t = Scheduler.TIME_INFINITY;
		}
		return schedule_absolute(t, e);
	}

	public String toString() {
		return "Scheduler";
	}

	public Scheduler(final Simulator s) {
		this.s = s;
		time = 0;
		events = new LLEventQueue();
	}
}
