package flowsim.core;

// 20.1.99: implemented fast-start

import java.util.Random;
import flowsim.util.Distribution;

/*
class DebugTraffic extends FlowImpl implements UnicastFlow
{
	Simulator s;
	static int total_running=0;
	
	public static UnicastFlowFactory flow_factory(final Simulator s) {
		return new UnicastFlowFactory() {
			public UnicastFlow create(Node from, Node to) {
				return new DebugTraffic(s,from,to);
			}
		};
	}

	public void start() {
		total_running++;
		print_totals();
	}

	public void stop() {
		total_running--;
		print_totals();
	}

	DebugTraffic(Simulator s, Node from, Node to) {
		super(from,0,0,0);
		this.s = s;
		//System.out.println(s.now()+":\tnew: "+from+"\t->\t"+to);
	}

	void print_totals() {
		System.out.println(s.now()+"\t"+total_running);
	}

	// not needed...
	public void next_node(Node child_node) {}
	public Node get_to() { return null; }
	public UnicastFlow get_parent() { return null; }
	public UnicastFlow get_child() { return null; }
	public void forward(LinkManager lm) {}
	public LinkManager get_iface() { return null; }
	public UnicastFlow add_child_same_node() { return null; }
	public void set_parent_bw(int bw) {}
}
*/

public class TrafficGenerator implements FlowMonitor
{
	final static boolean DEBUG=false;
	Simulator s;
	Topology t;
	Distribution arrival;
	Distribution duration;
	NodeChooser from_chooser;
	NodeChooser to_chooser;
	UnicastFlowFactory ff;
	EventQueueElement gfevent;
	boolean running;
	long started_flows;
	long current_flows;
	long blocked_flows;
	int minimal_current_flows;

	class GenerateFlowEvent implements Event {
		public void trigger() {
			generate_flow();
		}
	}

	class StopFlowEvent implements Event {
		Flow f;
		public void trigger() {
			debug("stopping flow");
			if((!running) || (current_flows>minimal_current_flows)) {
				current_flows--;
				f.stop();
			}
			else {
				// reschedule stop...
				s.schedule(10000,new StopFlowEvent(f)); 
			}
		}
		StopFlowEvent(Flow f) { this.f = f; }
	}

	void generate_flow() {
		if(running) {
			// select randomly source and destination
			Node from, to;
			int d;
			from = from_chooser.get();
			do {
				to = to_chooser.get();
			} while(from == to);

			d = (int) duration.rand();

			debug("generating flow ("+from+"->"+to+" d="+d+")");

			// create the flow
			Flow f = ff.create(from, to);
			f.set_flowid(10);

			// statistics
			f.attach_monitor(this);
			started_flows++;
			current_flows++;

			// start the flow
			f.start();
			s.schedule(d,new StopFlowEvent(f)); 
			schedule_next();
		}
	}

	void schedule_next() {
		int atime = (int) arrival.rand();
		debug("scheduling next flow at time "+atime);
		gfevent = s.schedule(atime,new GenerateFlowEvent());
	}

	public void start() {
		if(running == false) {
			running = true;
			schedule_next();
		}
	}

	public void stop() {
		debug("stopping");
		if(running == true && gfevent != null) {
			gfevent.remove();
		}
		running = false;
	}

	public void flow_started(Flow f) {}
	public void flow_stopped(Flow f) {}
	public void flow_changed_bw(Flow f, int was_bw) {}
	public void flow_changed_desired_bw(Flow f, int was_bw) {}
	public void flow_changed_available_bw(Flow f, int was_bw) {}

	public void flow_blocked(Flow f) {
		blocked_flows++;
		// System.out.println("Flow blocked (ds="+f.ds+")");
		if(f.get_flowid()!=10) {
			System.out.println("Alien flow blocked!");
		}
		//System.out.println("blocked="+blocked_flows);
	}

	public TrafficGenerator(Simulator s, Topology t, Distribution arrival,
		Distribution duration, NodeChooser from_chooser,
		NodeChooser to_chooser, UnicastFlowFactory ff,
		int minimal_current_flows)
	{
		this.s = s;
		this.t = t;
		this.arrival = arrival;
		this.duration = duration;
		this.ff = ff;
		this.from_chooser=from_chooser;
		this.to_chooser=to_chooser;
		this.minimal_current_flows = minimal_current_flows;
		running = false;
		gfevent = null;

		started_flows=0;
		current_flows=0;
		blocked_flows=0;
	}

	public void print_statistics() {
		System.out.println("started flows: "+started_flows);
		System.out.println("current flows: "+current_flows);
		System.out.println("blocked flows: "+blocked_flows);
		System.out.println("   block rate: "+(double)blocked_flows/started_flows);
	}

	public void init_stats() {
		started_flows=0;
		blocked_flows=0;
	}

	public int get_current_flows() {
		return (int) current_flows;
		
	}

	public double get_block_rate() {
		return (double)blocked_flows/started_flows;
	}

	final void debug(String str) {
		if(DEBUG) {
			System.err.println(s.now()+":\tTrafficGenerator:\t"+str);
		}
	}

	/*
	public static void main(String[] args) {
		Simulator s = new Simulator();
		Topology t = new Topology(s,12,4,1,null);
		LinkFactory lf =  FixedDelayLink.randomdelay_factory(s,10000,20000,0);
		LinkManagerFactory lmf = SimpleLinkManager.factory(s);
		
		int bw = 128*1024; // 128kbit/s
		t.add_duplex_link(lf,lmf,0,1,bw);
		t.add_duplex_link(lf,lmf,0,4,bw);
		t.add_duplex_link(lf,lmf,0,9,bw);
		t.add_duplex_link(lf,lmf,1,2,bw);
		t.add_duplex_link(lf,lmf,1,6,bw);
		t.add_duplex_link(lf,lmf,2,3,bw);
		t.add_duplex_link(lf,lmf,2,4,bw);
		t.add_duplex_link(lf,lmf,3,7,bw);
		t.add_duplex_link(lf,lmf,4,5,bw);
		t.add_duplex_link(lf,lmf,4,8,bw);
		t.add_duplex_link(lf,lmf,5,6,bw);
		t.add_duplex_link(lf,lmf,6,7,bw);
		t.add_duplex_link(lf,lmf,7,9,bw);
		t.add_duplex_link(lf,lmf,7,11,bw);
		t.add_duplex_link(lf,lmf,8,9,bw);
		t.add_duplex_link(lf,lmf,9,10,bw);
		t.add_duplex_link(lf,lmf,10,11,bw);

		TrafficGenerator tg = new TrafficGenerator(
			s,
			t,
			new ExponentialDistribution(1.0/100,0),
			new ExponentialDistribution(1.0/500000,0),
			new Random(10),
			DebugTraffic.flow_factory(s),
			4000
		);
			
		tg.start();

		// run simulation for some time
		s.run(10000000); // 1 s

		System.out.println();
	}
	*/
}
