package flowsim.dv;

import flowsim.core.*;

import java.io.*;
import java.util.*;

public class TestDV
{
	static Simulator s;
	static Topology t;
	static LinkFactory lf;
	static LinkManagerFactory lmf;

	public static void main(String[] args) {
		s = new Simulator();
		t = new Topology(s,6,3,1,"logs");
		t.add_services(DVRouterImpl.factory(s),0);
		//lf =  FixedDelayLink.randomdelay_factory(s,10000,50000,0);
		lf =  FixedDelayLink.factory(s,128000,20000);
		lmf = FairLinkManager.factory(s);

		// create links
		t.add_duplex_link(0,2,lf,lmf);
		t.add_duplex_link(1,2,lf,lmf);
		t.add_duplex_link(2,3,lf,lmf);
		t.add_duplex_link(3,4,lf,lmf);
		t.add_duplex_link(3,5,lf,lmf);

		// create nam log
		t.make_nam_log("testdv.nam");

		// run simulation for some time
		s.run(200000); // 200 ms

		print_routing_tables();

		// 200000

		RoutedFlow f1 = new FCBEFlowImpl(s,t.node(0),t.node(4),4,1,0);
		//RoutedFlow f1 = new UnicastFlowImpl(s,t.node(0),t.node(4),Integer.MAX_VALUE,0,1,0);
		f1.start();

		s.run(200000);

		// 400000

		RoutedFlow f2 = new FCBEFlowImpl(s,t.node(1),t.node(5),3,1,0);
		// RoutedFlow f2 = new UnicastFlowImpl(s,t.node(1),t.node(5),Integer.MAX_VALUE,0,1,0);
		f2.start();

		s.run(200000);

		// 600000

		f2.stop();

		s.run(200000);

		// 800000

		f1.stop();

		s.run(200000);

		t.flush_logs();
	}

	static void print_routing_tables() {
		// print routing tables
		for(int i=0; i<t.nodes(); i++) {
			System.out.println();
			System.out.println("Routing table for node nr. "+i);
			System.out.println("------------------------------");
			DVRouter r = (DVRouter) t.node(i).get_service(0);
			r.debug_table();
		}
	}
}
