package flowsim.slatr;

import flowsim.core.Node;
import flowsim.core.Simulator;

import com.sun.java.util.collections.Map;
import com.sun.java.util.collections.HashMap;

public final class DSNumbers
{
	final static boolean DEBUG = false;
	String dbg_string;
	final Simulator s;
	static final int DS_START = 10;

	Map available_ds_vector; // SLAId -> AutoNumbering

        final class Key {
                int peer;
		int dest_id;
                public boolean equals(Object obj) {
                        Key k = (Key) obj;
                        return (k.peer==peer && k.dest_id==dest_id);
                }
		public int hashCode() {
			return dest_id<<4+peer;
		}
                Key(int p, Node e) { peer=p; dest_id=e.get_id(); }
        }

	public int get_ds(int peer, Node dest) {
		Key key = new Key(peer,dest);
		AutoNumbering an = (AutoNumbering) available_ds_vector.get(key);
		if(an==null) {
			if(DEBUG) s.debug(this,"new AutoNumbering object for ["+peer+": "+dest+"]");
			an = new AutoNumbering(DS_START);
			available_ds_vector.put(key,an);
		}

		int ds = an.get();

		if(DEBUG) s.debug(this,"get -> ["+peer+": "+dest+", "+ds+"]");

		return ds;
	}

	public void release_ds(int peer, Node dest, int ds) {
		Key key = new Key(peer,dest);
		if(DEBUG) s.debug(this,"release ["+peer+": "+dest+", "+ds+"]");
		AutoNumbering an = (AutoNumbering) available_ds_vector.get(key);
		if(an==null) {
			s.error(this,"AutoNumbering object not found!");
			return;
		}
		an.release(ds);
	}

	public String toString() {
		if(dbg_string!=null) return "DSNumbers("+dbg_string+")";
		return "DSNumbers";
	}

	DSNumbers(Simulator s, String dbg_string) {
		this.s = s;
		this.dbg_string=dbg_string;
		available_ds_vector = new HashMap();
	}
}
