package flowsim.slatr;

import flowsim.core.*;

import com.sun.java.util.collections.Comparable;

final public class SLA implements Cloneable
{
	static final boolean DEBUG_STRING = true;

	public final Node dest;  // <----
	public int ds;             // <---- these two identify the SLA

	public final int bw;
	public final int max_delay;
	public long time;
	public long expiration;
	public long cost;

	// Not in protocol:

	public int peer;	// with multiple peers, this is also required to
				// identify the SLA.
	public int rbw;		// Used to keep track of how much the
				// taken_slas are used by given_slas.
	public int iface;	// iface of this peer (-1 if local)
	public long bid_expiration; // When does the bid guarantee expire?
	public boolean accepted; // Was an Accept message sent for this bid?
	public EventQueueElement expiration_event; // to remove it, if necessary
	public float utility;	// used by SLATUser
	public long gain;	// used by SLATUser
	public Object misc;	// for various data

	/** volume in bits */
	public long volume() {
		return volume(0);
	}

	public long volume(long now) {
		final long t = (now>time?now:time);
		//final long t = time;
		
		long   v = ((long)bw) * (expiration-t)/1000000;
		return v;
	}

	/** cost per volume */
	public float cost_volume(long now) {
		return (float) cost/volume(now);
	}

	public float cost_volume() {
		return cost_volume(0);
	}

	public float cost_time() {
		long   t = (expiration-time)/1000000;
		return (float) cost/t;
	}

	public SLA(Node dest, int ds, int bw, int max_delay, long cost,
		long time, long expiration)
	{
		this(dest,ds,bw,max_delay,cost,time,expiration,0);
	}

	public SLA(Node dest, int ds, int bw, int max_delay, long cost,
		long time, long expiration, int peer)
	{
		this.dest     = dest;
		this.ds         = ds;
		this.bw         = bw;
		this.max_delay  = max_delay;
		this.cost       = cost;
		this.expiration = expiration;
		this.rbw        = bw;
		this.accepted   = false;
		this.time       = time;
		this.peer       = peer;
		this.bid_expiration = 0; // no guarantees
	}

	// The following are implemented for SLA_List and DS_Map:

	public String toDebugString() {
		int rbwbw_percent = rbw*100/bw;
		int vcost = (int)(cost_volume()*10000);
		float vcostf = (float)vcost/100;
		return "["+peer+": "+dest+", "+ds+
			//" t="+time+" "+
			" "+bw/1000 + "kbps" +
			" "+max_delay/1000 + "ms" +
			" $" + cost +
			" "+volume()/1000+"kb"+
			" exp=" + expiration +
			" v$"+vcostf+
			" "+rbwbw_percent + "%" +
			"]";
	}

	public String toNormalString() {
		return "["+peer+": "+dest+", "+ds+"]";
	}

	public String toString() {
		// "[1->2 10000kb 5000us 10$]"
		if(DEBUG_STRING) return toDebugString();
		else return toNormalString();
	}

	public Object clone() throws CloneNotSupportedException
	{
		return super.clone();
	}

	// An SLA is identified by the (peer,ds) pair...
	public boolean equals(Object obj) {
		/*
		if(obj==null) return false;
		SLA b = (SLA) obj;
		if(peer == b.peer && dest==b.dest && ds == b.ds) return true;
		else return false;
		*/
		return this==obj;
	}

	public int hashCode() {
		return peer<<8+dest.get_id()<<4+ds;
	}
}
