package flowsim.slatr;

import flowsim.core.*;

import java.io.Serializable;

import com.sun.java.util.collections.Collections;
import com.sun.java.util.collections.Collection;
import com.sun.java.util.collections.TreeMap;
import com.sun.java.util.collections.Iterator;
import com.sun.java.util.collections.Map;
import com.sun.java.util.collections.SortedMap;
import com.sun.java.util.collections.Comparator;

final public class SLAList
{
	static final boolean DEBUG=false;
	final Simulator s;
	final String dbg_string;
	SortedMap table;

	public void put(final SLA sla) {
		SLAId slaid = new SLAId(sla.peer,sla.dest,sla.ds);
		if(DEBUG) s.debug(this,"put "+slaid);
		table.put(slaid,sla);
	}

	public SLA get(final int peer, final Node dest, final int ds) {
		return get(new SLAId(peer,dest,ds));
	}

	public SLA get(final SLAId slaid) {
		if(DEBUG) s.debug(this,"get "+slaid);
		return (SLA) table.get(slaid);
	}

	public boolean remove(final SLAId slaid, final Iterator it) {
		if(DEBUG) s.debug(this,"remove "+slaid);
		if(it!=null) it.remove();
		else {
			if(table.remove(slaid)==null) {
				s.error(this,"remove failed "+slaid);
				return false;
			}
		}
		return true;
	}

	public boolean remove(final int peer, final Node dest, final int ds, final Iterator it) {
		return remove(new SLAId(peer,dest,ds),it);
	}

	public boolean remove(final SLA sla) {
		return remove(sla,null);
	}

	public boolean remove(final SLA sla, final Iterator it) {
		SLAId slaid = new SLAId(sla);
		if(get(slaid)!=sla) return false;
		return remove(slaid,it);
	}

	public Collection slas() {
		return table.values();
	}

	public Collection search_dest(final Node dest) {
		if(dest==null) return slas();

		if(DEBUG) s.debug(this,"search_dest (dest_id="+dest.get_id()+")");

		SLAId a = new SLAId(Integer.MIN_VALUE,dest.get_id(),Integer.MIN_VALUE);
		SLAId b = new SLAId(Integer.MIN_VALUE,dest.get_id()+1,Integer.MIN_VALUE);

		SortedMap submap = table.subMap(a,b);

		if(DEBUG) s.debug(this,"submap size = "+submap.size()+" (total="+size()+")");

		return submap.values();
	}

	public int size() {
		return table.size();
	}

	public String toString() {
		if(dbg_string!=null) return "SLAList("+dbg_string+")";
		return "SLAList";
	}

	final int cmp(final int a, final int b) {
		if(a>b) return 1;
		else if(a<b) return -1;
		return 0;
	}

	final class EgressComparator implements Comparator, Serializable {
		public int compare(final Object o_a, final Object o_b) {
			SLAId a = (SLAId) o_a;
			SLAId b = (SLAId) o_b;

			final int cmp_a = cmp(a.dest_id,b.dest_id);
			if(cmp_a!=0) return cmp_a;

			final int cmp_b = cmp(a.peer,b.peer);
			if(cmp_b!=0) return cmp_b;
			
			return cmp(a.ds,b.ds);
		}
	}

	public SLAList(Simulator s, String dbg_string) {
		this.s = s;
		this.dbg_string = dbg_string;
		table = new TreeMap(new EgressComparator());
	}

	public SLAList(Simulator s) {
		this(s,null);
	}
}
