package flowsim.slatr;

import flowsim.core.*;
import flowsim.util.SearchIterator;

import com.sun.java.util.collections.Collection;
import com.sun.java.util.collections.Iterator;
import com.sun.java.util.collections.Map;
import com.sun.java.util.collections.HashMap;
import com.sun.java.util.collections.NoSuchElementException;
import com.sun.java.util.collections.UnsupportedOperationException;

final class SearchSLA extends  SearchIterator {
	final int min_bw;
	final int max_bw;

	final int min_rbw;
	final int max_rbw;

	final float min_rbwbw;
	final float max_rbwbw;

	final int min_delay;
	final int max_delay;

	final float min_cost;
	final float max_cost;

	final long min_exp;
	final long max_exp;

	// a <= x < b
	// assumption: x is positive...
	// if a or b are negative, don't test that relation
	boolean irange(int a, int x, int b) {
		if(a>0 && x<a) return false;
		if(b>0 && x>=b) return false;
		return true;
	}

	boolean lrange(long a, long x, long b) {
		if(a>0 && x<a) return false;
		if(b>0 && x>=b) return false;
		return true;
	}

	boolean frange(float a, float x, float b) {
		if(a>0 && x<a) return false;
		if(b>0 && x>=b) return false;
		return true;
	}

	public boolean match(final Object obj) {
		final SLA a = (SLA) obj;
		if(!irange(min_bw,   a.bw,        max_bw))    return false;
		if(!irange(min_rbw, a.rbw,        max_rbw))   return false;
		if(!irange(min_delay,a.max_delay, max_delay)) return false;
		if(!lrange(min_exp,  a.expiration,max_exp))   return false;
		if(!frange(min_cost,a.cost_volume(),max_cost))  return false;
		if(!frange(min_rbwbw,(float)a.rbw/a.bw,max_rbwbw)) return false;
		return true;
	}

	SearchSLA(final Iterator it,
			final int min_bw, final int max_bw,
			final int min_rbw, final int max_rbw,
			final float min_rbwbw, final float max_rbwbw,
			final int min_delay, final int max_delay,
			final float min_cost, final float max_cost,
			final long min_exp, final long max_exp)
	{
		super(it);

		this.min_bw=min_bw;
		this.max_bw=max_bw;

		this.min_rbw=min_rbw;
		this.max_rbw=max_rbw;

		this.min_rbwbw=min_rbwbw;
		this.max_rbwbw=max_rbwbw;

		this.min_delay=min_delay;
		this.max_delay=max_delay;

		this.min_cost=min_cost;
		this.max_cost=max_cost;

		this.min_exp=min_exp;
		this.max_exp=max_exp;
	}

	void add_str_list(StringBuffer sbuf, String str) {
		if(sbuf.length()!=0) sbuf.append(", ");
		sbuf.append(str);
	}

	public String toString() {
		StringBuffer sbuf = new StringBuffer();

		if(min_rbwbw>=0 || max_rbwbw>=0)
			add_str_list(sbuf,"bw="+min_bw+".."+max_bw);
		if(min_rbwbw>=0 || max_rbwbw>=0)
			add_str_list(sbuf,"rbwbw="+min_rbwbw+".."+max_rbwbw);
		if(min_delay>=0 || max_delay>=0)
			add_str_list(sbuf,"delay="+min_delay+".."+max_delay);
		if(min_cost>=0 || max_cost>=0)
			add_str_list(sbuf,"cost="+min_cost+".."+max_cost);
		if(min_exp>=0 || max_exp>=0)
			add_str_list(sbuf,"exp="+min_exp+".."+max_exp);

		return sbuf.toString();
	}
}

final class SLAListArrayIterator implements Iterator {
	final SLAList[] array;
	final Node dest;
	int current_index;
	Iterator current_iterator;

	final public boolean hasNext() {
		if(current_index>=array.length) return false;
		return true;
	}
	final public Object next() {
		if(!hasNext()) throw new NoSuchElementException();
		Object obj = current_iterator.next();
		next_iterator();
		return obj;
	}
	final public void remove() throws UnsupportedOperationException {
		throw new UnsupportedOperationException();
	}
	final private void next_iterator() {
		while(true) {
			if(current_iterator!=null && current_iterator.hasNext()) return;
			current_index++;
			if(current_index>=array.length) return;
			SLAList sl = array[current_index];
			current_iterator=sl.search_dest(dest).iterator();
		}
	}
	SLAListArrayIterator(final SLAList[] array, final Node dest) {
		this.array = array;
		this.dest = dest;
		current_index=-1;
		current_iterator=null;
		next_iterator();
	}
}

public final class SLAMap
{
	final Simulator s;
	static final boolean DEBUG=false;
	Map given_to_taken;
	String dbg_string;

	int peers;

	SLAList[] given_bids; // sent
	SLAList[] given_slas;

	SLAList taken_bids; // received
	SLAList taken_slas;

	public void map_given_to_taken(SLA given, SLA taken) {
		if(DEBUG) {
			s.debug(this,"mapping "+given+" to "+taken);
			if(get_taken_from_given(given)!=null) {
				s.error(this,"Trying to remap "+given+" to "+taken);
			}
		}
		given_to_taken.put(given, taken);
	}

	public SLA get_taken_from_given(int peer, Node dest, int ds) {
		SLA given = get_given_sla(peer,dest,ds);
		return get_taken_from_given(given);
	}

	public SLA get_taken_from_given(SLA given) {
		if(DEBUG) s.debug(this,"get taken from given "+given);
		if(given!=null) {
			SLA taken = (SLA) given_to_taken.get(given);
			if(DEBUG) s.debug(this,"->"+taken);
			return taken;
		}
		else {
			s.error(this,"get_taken_from_given: given=null!");
			return null;
		}
	}

	public boolean remove_taken_sla(SLA taken) {
		return remove_taken_sla(taken,null);
	}

	public boolean remove_taken_sla(SLA taken, Iterator it) {
		// if it!=null use it.remove()

		if(DEBUG) s.debug(this,"remove_taken_sla: "+taken);

		//if(taken.peer<0 || taken.peer>=peers) return;
		return taken_slas.remove(taken,it);
	}

	public boolean remove_taken_bid(SLA taken) {
		return remove_taken_bid(taken,null);
	}

	public boolean remove_taken_bid(SLA taken, Iterator it) {
		if(DEBUG) s.debug(this,"remove_taken_bid: "+taken);

		//if(taken.peer<0 || taken.peer>=peers) return;
		return taken_bids.remove(taken,it);
	}

	public boolean remove_given_sla(SLA given) {
		return remove_given_sla(given,null);
	}

	public boolean remove_given_sla(SLA given, Iterator it) {
		if(DEBUG) s.debug(this,"remove_given_sla: "+given);

		//if(given.peer<0 || given.peer>=peers) return;
		if(given_slas[given.peer].remove(given,it)) {
			given_to_taken.remove(given);
			return true;
		}
		return false;

	}

	public boolean remove_given_bid(SLA given) {
		return remove_given_bid(given,null);
	}

	public boolean remove_given_bid(SLA given, Iterator it) {
		if(DEBUG) s.debug(this,"remove_given_bid: "+given);

		//if(given.peer<0 || given.peer>=peers) return;
		if(given_bids[given.peer].remove(given,it)) {
			given_to_taken.remove(given);
			return true;
		}
		return false;
	}

	public SLA move_taken_bid_to_sla(int peer, Node dest, int ds) {
		SLA bid = get_taken_bid(peer,dest,ds);
		if(bid!=null) move_taken_bid_to_sla(bid);
		else {
			s.debug(this,"move_taken_bid_to_sla: bid==null");
		}
		return bid;
	}

	public void move_taken_bid_to_sla(SLA bid) {
		if(DEBUG) s.debug(this,"move_taken_bid_to_sla: "+bid);
		//if(bid.peer<0 || bid.peer>=peers) return;
		taken_bids.remove(bid,null);
		taken_slas.put(bid);
	}

	public SLA move_given_bid_to_sla(int peer, Node dest, int ds) {
		SLA bid = get_given_bid(peer,dest,ds);
		if(bid!=null) move_given_bid_to_sla(bid);
		return bid;
	}

	public void move_given_bid_to_sla(SLA bid) {
		if(DEBUG) s.debug(this,"move_given_bid_to_sla: "+bid);
		if(given_bids[bid.peer].remove(bid,null)) {
			given_slas[bid.peer].put(bid);
		}
	}

	public void put_taken_sla(SLA sla) {
		if(DEBUG) s.debug(this,"put_taken_sla");
		//if(sla.peer<0 || sla.peer>=peers) return;
		taken_slas.put(sla);
	}

	public void put_taken_bid(SLA bid) {
		if(DEBUG) s.debug(this,"put_taken_bid: "+bid);
		//if(bid.peer<0 || bid.peer>=peers) return;
		taken_bids.put(bid);
	}

	public void put_given_sla(SLA sla) {
		if(DEBUG) s.debug(this,"put_given_sla");
		//if(sla.peer<0 || sla.peer>=peers) return;
		given_slas[sla.peer].put(sla);
	}

	public void put_given_bid(SLA bid) {
		if(DEBUG) s.debug(this,"put_given_bid: "+bid);
		//if(bid.peer<0 || bid.peer>=peers) return;
		given_bids[bid.peer].put(bid);
	}

	public SLA get_taken_sla(final int peer, final Node dest, final int ds) {
		//if(peer<0 || peer>=peers) return null;
		return taken_slas.get(peer, dest, ds);
	}

	public SLA get_given_sla(final int peer, final Node dest, final int ds) {
		//if(peer<0 || peer>=peers) return null;
		return given_slas[peer].get(peer, dest, ds);
	}

	public SLA get_taken_bid(final int peer, final Node dest, final int ds) {
		//if(peer<0 || peer>=peers) return null;
		return taken_bids.get(peer, dest,ds);
	}

	public SLA get_given_bid(final int peer, final Node dest, final int ds) {
		//if(peer<0 || peer>=peers) return null;
		return given_bids[peer].get(peer, dest,ds);
	}

	int abs(final int x) { if(x<0) return -x; return x; }

	public Iterator taken_slas_to_dest(final Node dest)
	{
		if(DEBUG) s.debug(this,"searching ts for sla to dest "+dest); 
		return taken_slas.search_dest(dest).iterator();
	}

	public Iterator taken_bids_to_dest(final Node dest)
	{
		if(DEBUG) s.debug(this,"searching ts for sla to dest "+dest); 
		return taken_bids.search_dest(dest).iterator();
	}

	public Iterator given_bids_to_dest(final Node dest)
	{
		if(DEBUG) s.debug(this,"searching gb for bids to dest "+dest); 
		return new SLAListArrayIterator(given_bids,dest);
	}

	public Iterator given_slas_to_dest(Node dest)
	{
		if(DEBUG) s.debug(this,"searching gs for slas to dest "+dest); 
		return new SLAListArrayIterator(given_slas,dest);
	}

	public Iterator search_given_bids(final int peer, final Node dest,
			final int min_bw, final int max_bw,
			final int min_rbw, final int max_rbw,
			final float min_rbwbw, final float max_rbwbw,
			final int min_delay, final int max_delay,
			final float min_cost, final float max_cost,
			final long min_exp, final long max_exp)
	{
		Iterator it = given_bids[peer].search_dest(dest).iterator();
		SearchSLA si = new SearchSLA(it,
				min_bw, max_bw,
				min_rbw, max_rbw,
				min_rbwbw, max_rbwbw,
				min_delay, max_delay,
				min_cost, max_cost,
				min_exp, max_exp);
		if(DEBUG) s.debug(this,"searching "+peer+",gb for ["+dest+", "+si+"]");
		if(DEBUG) s.debug(this,"-> hasNext()? "+si.hasNext());
		return si;
	}

	public Iterator search_given_slas(final int peer, final Node dest,
			final int min_bw, final int max_bw,
			final int min_rbw, final int max_rbw,
			final float min_rbwbw, final float max_rbwbw,
			final int min_delay, final int max_delay,
			final float min_cost, final float max_cost,
			final long min_exp, final long max_exp)
	{
		Iterator it = given_slas[peer].search_dest(dest).iterator();
		SearchSLA si = new SearchSLA(it,
				min_bw, max_bw,
				min_rbw, max_rbw,
				min_rbwbw, max_rbwbw, 
				min_delay, max_delay,
				min_cost, max_cost,
				min_exp, max_exp);
		if(DEBUG) s.debug(this,"searching "+peer+",gs for ["+dest+", "+si+"]");
		return si;
	}

	public Iterator search_taken_slas(final Node dest,
			final int min_bw, final int max_bw,
			final int min_rbw, final int max_rbw,
			final float min_rbwbw, final float max_rbwbw,
			final int min_delay, final int max_delay,
			final float min_cost, final float max_cost,
			final long min_exp, final long max_exp)
	{
		Iterator it = taken_slas.search_dest(dest).iterator();
		SearchSLA si = new SearchSLA(it,
				min_bw, max_bw,
				min_rbw, max_rbw,
				min_rbwbw, max_rbwbw, 
				min_delay, max_delay,
				min_cost, max_cost,
				min_exp, max_exp);
		if(DEBUG) s.debug(this,"searching ts for ["+dest+", "+si+"]");
		return si;
	}

	public Iterator search_taken_bids(final Node dest,
			final int min_bw, final int max_bw,
			final int min_rbw, final int max_rbw,
			final float min_rbwbw, final float max_rbwbw,
			final int min_delay, final int max_delay,
			final float min_cost, final float max_cost,
			final long min_exp, final long max_exp)
	{
		Iterator it = taken_bids.search_dest(dest).iterator();
		SearchSLA si = new SearchSLA(it,
				min_bw, max_bw,
				min_rbw, max_rbw,
				min_rbwbw, max_rbwbw, 
				min_delay, max_delay,
				min_cost, max_cost,
				min_exp, max_exp);
		if(DEBUG) s.debug(this,"searching tb for ["+dest+", "+si+"]");
		return si;
	}

	public Collection given_slas(int peer) {
		return given_slas[peer].slas();
	}

	public Collection given_bids(int peer) {
		return given_bids[peer].slas();
	}

	public Collection taken_slas() {
		return taken_slas.slas();
	}

	public Collection taken_bids() {
		return taken_bids.slas();
	}

	public String toString() {
		if(dbg_string!=null) return "SLAMap("+dbg_string+")";
		return "SLAMap";
	}

	public SLAMap(Simulator s, int peers) {
		this(s,peers,null);
	}

	public SLAMap(Simulator s, int peers, String dbg_string) {
		this.s = s;
		this.peers = peers;
		this.dbg_string = dbg_string;

		given_to_taken = new HashMap();

		given_slas = new SLAList[peers];
		given_bids = new SLAList[peers];
		taken_slas = new SLAList(s,dbg_string+",ts");
		taken_bids = new SLAList(s,dbg_string+",tb");

		for(int i=0; i<peers; i++) {
			given_slas[i] = new SLAList(s,dbg_string+","+i+",gs");
			given_bids[i] = new SLAList(s,dbg_string+","+i+",gb");
		}
	}
}
