package flowsim.slatr;

import flowsim.core.*;

/**
 * Accept message of the SLA Trading Protocol (SLATP).
 * @see SLATP_Bid
 * @see SLATP_Confirm
 */
public final class SLATP_Accept extends SLATP_Message
{
	final int  sla_ds;
	final Node sla_dest;

	public int get_sla_ds() {
		return sla_ds;
	}

	public Node get_sla_dest() {
		return sla_dest;
	}

	/**
	 * Send a SLATP_Accept message to a SLAT-peer which can be local or on another
	 * node. If the destination is on another node, a PacketImpl flow is
	 * created and started.
	 * @param s simulator object
	 * @param from buyer
	 * @param to seller
	 * @param sla_dest dest node of the SLA in question.
	 * @param sla_ds ds of the SLA in question. 
	 * @see PacketImpl
	 */
	public static void send(Simulator s, SLABuyer from, SLASeller to,
		Node sla_dest, int sla_ds)
	{
		SLATP_Message msg = new SLATP_Accept(from,to,sla_dest,sla_ds);
		send_message(s,from,to,msg);
	}

	public int size() { return 20; }

	public void arrived() {
		SLASeller seller = (SLASeller) to;
		seller.receive_accept(this);
	}

	SLATP_Accept(SLABuyer from, SLASeller to, Node sla_dest, int sla_ds)
	{
		super(from,to);
		this.sla_ds     = sla_ds;
		this.sla_dest = sla_dest;
	}
}
