package flowsim.slatr;

import flowsim.core.*;

/**
 * Ask message of the SLA Trading Protocol (SLATP).
 * @see SLATP_Bid
 */
public class SLATP_Ask extends SLATP_Message
{
	public final Node dest;
	public final int bw;
	public final int max_delay;
	public final long expiration;

	/**
	 * Send a SLATP_Ask message to a SLAT-peer which can be local or on another
	 * node. If the destination is on another node, a PacketImpl flow is
	 * created and started.
	 * @param s simulator object
	 * @param from buyer
	 * @param to seller
	 * @param dest wanted dest point
	 * @param bw wanted (ideal) bandwidth
	 * @param max_delay wanted (ideal) maximal delay
	 * @param expiration wanted (ideal) expiration
	 * @see PacketImpl
	 */
	public static void send(Simulator s, SLABuyer from, SLASeller to,
		Node dest, int bw, int max_delay, long expiration)
	{
		SLATP_Message msg = new SLATP_Ask(from,to,dest,bw,max_delay,expiration);
		send_message(s,from,to,msg);
	}

	public int size() { return 20; }

	public void arrived() {
		SLASeller seller = (SLASeller) to;
		seller.receive_ask(this);
	}

	SLATP_Ask(SLABuyer from, SLASeller to, Node dest, int bw, int max_delay, long expiration)
	{
		super(from,to);

		this.dest     = dest;
		this.bw         = bw;
		this.max_delay  = max_delay;
		this.expiration = expiration;
	}
}
