package flowsim.slatr;

import flowsim.core.*;

/**
 * Bid message of the SLA Trading Protocol (SLATP).
 * @see SLATP_Ask
 * @see SLATP_Accept
 */
public final class SLATP_Bid extends SLATP_Message
{
	SLA sla;

	public SLA get_sla() {
		return sla;
	}

	/**
	 * Send a SLATP_Bid message to a SLAT-peer which can be local or on another
	 * node. If the destination is on another node, a PacketImpl flow is
	 * created and started.
	 * @param s simulator object
	 * @param from seller
	 * @param to buyer
	 * @param sla SLA proposed
	 * @see PacketImpl
	 */
	public static void send(Simulator s, SLASeller from, SLABuyer to, SLA sla)
	{
		SLATP_Message msg = new SLATP_Bid(from,to,sla);
		send_message(s,from,to,msg);
	}

	public final int size() { return 30; }

	public void arrived() {
		SLABuyer buyer = (SLABuyer) to;
		buyer.receive_bid(this);
	}

	public SLATP_Bid(Service from, Service to, SLA sla) {
		super(from,to);
		try {
			this.sla = (SLA) sla.clone();
		}
		catch(CloneNotSupportedException e) {
			this.sla = null;
		}
	}
}
