package flowsim.slatr;

import flowsim.core.*;

abstract class SLATP_Message implements Message
{
	final Service from;
	final Service to;

	public Service get_from() {
		return from;
	}

	public Service get_to() {
		return to;
	}

	static void send_message_to_iface(Simulator s, Node from, int iface, Message msg)
	{
		LinkManager lm = from.get_iface(iface);
		Packet packet = new NextHopPacket(s,from,lm,msg,FlowImpl.DS_BEST_EFFORT);
		packet.start();
	}

	static void send_message(Simulator s, Service from, Service to, SLATP_Message msg)
	{
		Node from_node = from.get_node();
		Node to_node = to.get_node();
		if(from_node != to_node) {
			int iface = from_node.get_iface(to_node);
			if(iface<0) {
				System.err.println("SLATP_Message: node "+to+"isn't peer of "+from);
				return;
			}
			
			send_message_to_iface(s,from.get_node(),iface,msg);
		}
		else {
			// It's local!
			msg.arrived();
		}
	}

	public final void arrived(Node to_node) {
		// Check node
		if(to.get_node()!=to_node) {
			System.err.println("SLATP_Message: Huston, we have a problem:"+
				" to.get_node()!=to_node");
			return;
		}
		arrived();
	}

	abstract void arrived();

	SLATP_Message(Service from, Service to) {
		this.from       = from;
		this.to         = to;
	}
}
