package flowsim.slatr;

import flowsim.core.*;

/**
 * Reject message of the SLA Trading Protocol (SLATP).
 * @see SLATP_Accept
 * @see SLATP_Reject
 */
public class SLATP_Reject extends SLATP_Message
{
	final int  sla_ds;
	final Node sla_dest;

	public int get_sla_ds() {
		return sla_ds;
	}

	public Node get_sla_dest() {
		return sla_dest;
	}

	/**
	 * Send a SLATP_Reject message to a SLAT-peer which can be local or on another
	 * node. If the destination is on another node, a PacketImpl flow is
	 * created and started.
	 * @param s simulator object
	 * @param from seller
	 * @param to buyer
	 * @param sla_dest dest node of the SLA in question.
	 * @param sla_ds ds of the SLA in question. 
	 * @see PacketImpl
	 */
	public static void send(Simulator s, SLASeller from, SLABuyer to,
		Node sla_dest, int sla_ds)
	{
		SLATP_Message msg = new SLATP_Reject(from,to,sla_dest,sla_ds);
		send_message(s,from,to,msg);
	}

	public final int size() {
		return 20;
	}

	public void arrived() {
		SLABuyer buyer = (SLABuyer) to;
		buyer.receive_reject(this);
	}

	SLATP_Reject(Service from, Service to, Node sla_dest, int sla_ds)
	{
		super(from,to);
		this.sla_ds     = sla_ds;
		this.sla_dest = sla_dest;
	}
}
