package flowsim.slatr;

import flowsim.core.*;
import flowsim.dv.*;

import java.util.Random;

public class TestFTP
{
	static final int SIZE_START =  10000;
	static final int SIZE_END   = 300000;
	static final int SIZE_STEP  =  10000;
	/*
	static final int SIZE_STEPS = 10;
	static final int SIZE_STEP  = (SIZE_END-SIZE_START)/SIZE_STEPS;
	*/

	public static void main(String[] args) {
		/*
		for(int size=SIZE_START; size<=SIZE_END; size+=SIZE_STEP) {
			TestFTPtest test = new TestFTPtest(size);
			test.test();
		}
		*/
		int bw = 1500000;
		TestFTPtest test = new TestFTPtest(bw);
		float utility = test.test();
		System.out.println(bw+"\t"+utility);
	}
}

final class TestFTPtest
{
	final Simulator s;
	final Topology t;
	final Random r;
	LinkFactory lf;
	LinkManagerFactory lmf;
	static final int NODES = 12;
	static final int MAX_NEIGHBOURS = 4;
	static final int DVROUTER_PORT = 0;
	static final int MAX_ROUTERS = 1;
	static final int USER_PORT = MAX_ROUTERS;
	static final int MAX_USERS=5;
	final int[] user_port;
	final int size;

	TestFTPtest(int size) {
		this.size = size;
		r = new Random(12444);
		s = new Simulator();
		s.set_verbose(false);
		t = new Topology(s,NODES,MAX_NEIGHBOURS,MAX_ROUTERS+MAX_USERS,null);

		t.add_services(DVRouterImpl.factory(s),DVROUTER_PORT);

		user_port = new int[NODES];
		for(int n=0; n<NODES; n++) {
			user_port[n]=USER_PORT;
		}
	}

	void add_user_ftp(int n, long budget,
		long budget_incr, int flow_id, int transfers_per_second,
		int average_size)
	{
		int port = user_port[n]++;
		NodeChooser nc = t.get_random_node_chooser(r);
		new FTPUser(s, t.node(n), port, nc, budget,
			budget_incr, flow_id, DVROUTER_PORT,
			transfers_per_second, average_size, r);
	}

	void add_users() {
		add_user_ftp(0,10000,1000,0,100,size);
		add_user_ftp(1,10000,1000,1,100,size);
		add_user_ftp(2,10000,1000,2,100,size);
	}

	float test() {
		// create links
		lf =  FixedDelayLink.randomdelay_factory(s,10000000,10000,20000,0);
		lmf = FairLinkManager.factory(s);

		t.add_duplex_link(0,1,lf,lmf);
		t.add_duplex_link(0,4,lf,lmf);
		t.add_duplex_link(0,9,lf,lmf);
		t.add_duplex_link(1,2,lf,lmf);
		t.add_duplex_link(1,6,lf,lmf);
		t.add_duplex_link(2,3,lf,lmf);
		t.add_duplex_link(2,4,lf,lmf);
		t.add_duplex_link(3,7,lf,lmf);
		t.add_duplex_link(4,5,lf,lmf);
		t.add_duplex_link(4,8,lf,lmf);
		t.add_duplex_link(5,6,lf,lmf);
		t.add_duplex_link(6,7,lf,lmf);
		t.add_duplex_link(7,9,lf,lmf);
		t.add_duplex_link(7,11,lf,lmf);
		t.add_duplex_link(8,9,lf,lmf);
		t.add_duplex_link(9,10,lf,lmf);
		t.add_duplex_link(10,11,lf,lmf);

		// run simulation for some time
		s.run(1*1000*1000); // 1 seconds

		add_users();

		// run simulation for some time
		s.run(5*1000*1000); // 5 seconds

		User user = (User) t.node(0).get_service(USER_PORT);
		return user.get_utility();
	}
}
