
package flowsim.slatr;

import flowsim.core.*;
import flowsim.dv.*;

import java.util.Random;
import java.io.*;

final public class TestImpl
{
	public static void main(String[] args) {
		int bw = Integer.parseInt(args[0]);
		int test_what = Integer.parseInt(args[1]);
		TestImplRun test = new TestImplRun(bw,test_what);
		float utility = test.run();
		System.out.println(bw+"\t"+utility);
	}
}

final class TestImplRun
{
	Simulator s;
	Topology t;
	LinkFactory lf;
	LinkManagerFactory lmf;
	Random r;
	int nodes;

	static final int     RUN_SECONDS = 5;
	static final boolean TEST_SLAT   = true; // if false: test DV
	static final boolean TRACE_NAM   = false;
	static final boolean TRACE_BW    = false;

	static final int GRID_SIZE_X = 4;
	static final int GRID_SIZE_Y = 4;
	static final int MAX_NEIGHBOURS = 8;
	static final int ROUTER_PORT = 0;
	static final int MAX_ROUTERS = 1;
	static final int USER_PORT = MAX_ROUTERS;
	static final int MAX_USERS=2;

	static final int CALLS_PER_SECOND   = 500;
	static final int CALL_BW            = 16000;
	static final int USER_BUDGET        = 100000;
	static final int ROUTER_BUDGET      = 10000000;

	static final boolean PERIODIC_DEBUG     = false;
	static final long PERIODIC_DEBUG_PERIOD = 1000000;
	static final long LOG_UTILITY_PERIOD    = 200000;
	static final long NAM_CYCLE_TIME        = 10000;

	final int test_what;
	static final String TEST_NODE = "5";

	float last_max_utility=0, last_utility=0;
	PrintStream logutility;
	PrintStream logmaxutility;
	File logs_dir;
	int[] user_port;
	int bw;

	final class SLATRouterFactory implements ServiceFactory
	{
		public Service create(Node node, int port) {
			if(node.get_name().equals(TEST_NODE)) {
				switch(test_what) {
					case 0: return new
						SLATRouterGreedy(s,node,port,MAX_USERS,
								ROUTER_BUDGET,0,r);
					case 1: return new
						SLATRouterTrendy(s,node,port,MAX_USERS,
								ROUTER_BUDGET,0,r);
					case 2: return new
						SLATRouterProfitable(s,node,port,MAX_USERS,
								ROUTER_BUDGET,0,r);
					case 3: return new
						SLATRouterLazy(s,node,port,MAX_USERS,
								ROUTER_BUDGET,0,r);
				}
			}

			return new SLATRouterProfitable(s,node,port,MAX_USERS,
						ROUTER_BUDGET,0,r);
		}
	}

	final class LogUtility implements Event {
		public void trigger() { log_utility(); }
	}

	TestImplRun(int bw,int test_what) {
		this.bw = bw;
		this.test_what=test_what;
	}

	void log_utility() {
		User u = (User) t.node(0).get_service(USER_PORT);
		if(u!=null) {
			float utility     = u.get_utility();
			float max_utility = u.get_max_utility();

			long now = s.now();
			logutility.println(now+"\t"+(utility-last_utility));
			logmaxutility.println(now+"\t"+(max_utility-last_max_utility));

			last_utility = utility;
			last_max_utility = max_utility;
		}
		s.schedule(LOG_UTILITY_PERIOD,new LogUtility());
	}

	Topology create_grid(int x_size, int y_size, LinkFactory lf, LinkManagerFactory lmf,
			File logs_dir)
	{
		int nodes = x_size*y_size;

		String logs_dir_str;
		if(TRACE_BW) logs_dir_str=logs_dir.toString();
		else         logs_dir_str=null;
		Topology t = new Topology(s,nodes,MAX_NEIGHBOURS,
			MAX_ROUTERS+MAX_USERS, logs_dir_str);

		// --- this should be moved out of that function...
		if(TEST_SLAT) {
			ServiceFactory sf = new SLATRouterFactory();
			t.add_services(sf,ROUTER_PORT);
		}
		else {
			t.add_services(DVRouterImpl.factory(s),ROUTER_PORT);
		}
		// ---

		for(int y=0; y<y_size; y++) {
			for(int x=0; x<x_size; x++) {
				// right link
				if(x<x_size-1) {
					t.add_duplex_link(y*x_size+x,y*x_size+x+1,lf,lmf);
				}

				// left-down link
				if(x>0 && y<y_size-1) {
					t.add_duplex_link(y*x_size+x,(y+1)*x_size+(x-1),lf,lmf);
				}

				// down link
				if(y<y_size-1) {
					t.add_duplex_link(y*x_size+x,(y+1)*x_size+x,lf,lmf);
				}

				// right-down link
				if(x<x_size-1 && y<y_size-1) {
					t.add_duplex_link(y*x_size+x,(y+1)*x_size+x+1,lf,lmf);
				}

			}
		}

		return t;
	}

	void init() {
		r = new Random(12444);
		s = new Simulator();
		s.set_verbose(true);
		logs_dir = new File("logs-impl-"+test_what+"-"+bw);
		logs_dir.mkdir();

		// create grid topology (nodes, links and linkmanagers)
		nodes = GRID_SIZE_X*GRID_SIZE_Y;
		lf =  FixedDelayLink.factory(s,10000000,10000);
		lmf = FairLinkManager.factory(s);
		t = create_grid(GRID_SIZE_X, GRID_SIZE_Y, lf, lmf, logs_dir);

		user_port = new int[nodes];
		for(int n=0; n<nodes; n++) {
			user_port[n]=USER_PORT;
		}

		try {
			logutility = new PrintStream(
					new FileOutputStream(logs_dir.toString()+"/utility"));
			logmaxutility = new PrintStream(
					new FileOutputStream(logs_dir.toString()+"/max_utility"));

			log_utility();
		}
		catch(IOException e) {
			logutility=logmaxutility=null;
		}
	}

	void add_user_voice(int n, long budget,
			long budget_incr, int calls_per_second,
			int mean_duration,NodeChooser nodechooser, int flowid)
	{
		int port = user_port[n]++;
		if(TEST_SLAT) {
			new SLATUserVoice(s, t, t.node(n), port, budget, budget_incr,
					ROUTER_PORT, calls_per_second, mean_duration, r,
					nodechooser,flowid,10);
		}
		else {
			new UserVoice(s, t, t.node(n), port, 
					ROUTER_PORT, calls_per_second, mean_duration, r,
					nodechooser,flowid);
		}
	}

	void add_users() {
		long size = bw/CALLS_PER_SECOND;
		int duration = (int)(size*1000000/CALL_BW);
		//int budget_incr = 10*CALLS_PER_SECOND+bw;
		int budget_incr = 100000000;

		s.debug(this,"size="+size+", duration="+duration);

		NodeChooser nodechooser1 = new NodeChooser() {
			public Node get() {
				return t.node(15);
			}
		};

		add_user_voice(0,USER_BUDGET,budget_incr,
				CALLS_PER_SECOND,duration,nodechooser1,3);

		/*
		   NodeChooser nodechooser2 = new NodeChooser() {
		   public Node get() {
		   return t.node(6);
		   }
		   };

		   add_user_voice(5,USER_BUDGET,budget_incr,
		   CALLS_PER_SECOND,duration,nodechooser2,4);
		 */
	}

	void print_routing_tables_slatr(PrintStream ps) {
		for(int i=0; i<t.nodes(); i++) {
			ps.println();
			SLATRouter r = (SLATRouter) t.node(i).get_service(ROUTER_PORT);
			r.debug_table(ps);
		}
	}

	class PeriodicDebugEvent implements Event {
		public void trigger() { periodic_debug(); }
	}

	void periodic_debug() {
		try {
			FileOutputStream fos = new FileOutputStream("pdebug/pdebug-"+s.now());
			PrintStream ps = new PrintStream(fos);
			print_routing_tables_slatr(ps);
			s.schedule(PERIODIC_DEBUG_PERIOD,new PeriodicDebugEvent());
		}
		catch(IOException e) {}
	}

	public float run() {
		init();

		// create nam log
		if(TRACE_NAM) t.make_nam_log(logs_dir.toString()+"/testimpl.nam",NAM_CYCLE_TIME,true);

		if(PERIODIC_DEBUG && TEST_SLAT) periodic_debug();

		s.run(1*1000*1000); // 1 second

		add_users();

		s.run(RUN_SECONDS*1000*1000);

		//print_routing_tables_dv();
		print_routing_tables_slatr(System.out);

		t.flush_logs();

		User user = (User) t.node(0).get_service(USER_PORT);

		// System.out.println("max="+user.get_max_utility());

		return user.get_utility()/user.get_max_utility();
	}
}
