
package flowsim.slatr;

import flowsim.core.*;
import flowsim.dv.*;

import java.util.Random;
import java.io.*;

final public class TestLoadBalancing
{
	public static void main(String[] args) {
		int bw = Integer.parseInt(args[0]);
		TestLoadBalancingRun test = new TestLoadBalancingRun(bw);
		float utility = test.run();
		System.out.println(bw+"\t"+utility);
	}
}

final class TestLoadBalancingRun
{
	Simulator s;
	Topology t;
	LinkFactory lf;
	LinkManagerFactory lmf;
	Random r;
	static final int NODES = 12;
	static final int MAX_NEIGHBOURS = 4;
	static final int SLATROUTER_PORT = 0;
	static final int MAX_ROUTERS = 1;
	static final int USER_PORT = MAX_ROUTERS;
	static final int MAX_USERS=1;
	int[] user_port;
	int bw;

	static final int CALLS_PER_SECOND = 1000;
	static final int CALL_BW = 16000;
	static final int USER_BUDGET        = 100000;
	static final int ROUTER_BUDGET      = 1000000;
	static final int ROUTER_BUDGET_INCR = 1000000;

	static final long PERIODIC_DEBUG_PERIOD = 50000;

	static final long LOG_UTILITY_PERIOD = 50000;
	float last_max_utility=0, last_utility=0;
	PrintStream logutility;
	PrintStream logmaxutility;

	final class SLATRouterFactory implements ServiceFactory
	{
		static final float TRENDY = (float) 0;
		static final float PROFI  = (float) 1;

		public Service create(Node node, int port) {
			float sel = r.nextFloat();

			if(sel<TRENDY) {
				return new
					SLATRouterTrendy(s,node,port,MAX_USERS,
							ROUTER_BUDGET,0,r);
			}
			else {
				return new
					SLATRouterProfitable(s,node,port,MAX_USERS,
							ROUTER_BUDGET,0,r);
			}
		}
	}

	final class LogUtility implements Event {
		public void trigger() { log_utility(); }
	}

	TestLoadBalancingRun(int bw) {
		this.bw = bw;
	}

	void log_utility() {
		User u = (User) t.node(0).get_service(USER_PORT);
		if(u!=null) {
			float utility     = u.get_utility();
			float max_utility = u.get_max_utility();

			long now = s.now();
			logutility.println(now+"\t"+(utility-last_utility));
			logmaxutility.println(now+"\t"+(max_utility-last_max_utility));

			last_utility = utility;
			last_max_utility = max_utility;
		}
		s.schedule(LOG_UTILITY_PERIOD,new LogUtility());
	}

	void init() {
		r = new Random(12444);
		s = new Simulator();
		s.set_verbose(true);
		File logs_dir = new File("logs-lb1-"+bw);
		logs_dir.mkdir();
		t = new Topology(s,NODES,MAX_NEIGHBOURS,MAX_ROUTERS+MAX_USERS,logs_dir.toString());
		//t = new Topology(s,NODES,MAX_NEIGHBOURS,MAX_ROUTERS+MAX_USERS,null);

		/*
		t.add_services(DVRouterImpl.factory(s),DVROUTER_PORT);
		*/

		ServiceFactory sf = new SLATRouterFactory();

		t.add_services(sf,SLATROUTER_PORT);

		user_port = new int[NODES];
		for(int n=0; n<NODES; n++) {
			user_port[n]=USER_PORT;
		}
		
		try {
			logutility = new PrintStream(
					new FileOutputStream(logs_dir.toString()+"/utility"));
			logmaxutility = new PrintStream(
					new FileOutputStream(logs_dir.toString()+"/max_utility"));
			
			log_utility();
		}
		catch(IOException e) {
			logutility=logmaxutility=null;
		}
	}

	void add_user_voice(int n, long budget,
		long budget_incr, int calls_per_second,
		int mean_duration,NodeChooser nodechooser)
	{
		int port = user_port[n]++;
		new SLATUserVoice(s, t, t.node(n), port, budget, budget_incr,
			SLATROUTER_PORT, calls_per_second, mean_duration, r,
			nodechooser,0,10);
	}

	void add_users() {
		long size = bw/CALLS_PER_SECOND;
		int duration = (int)(size*1000000/CALL_BW);
		//int budget_incr = 10*CALLS_PER_SECOND+bw;
		int budget_incr = 100000000;

		s.debug(this,"size="+size+", duration="+duration);

		NodeChooser nodechooser = new NodeChooser() {
			public Node get() {
				return t.node(11);
			}
		};
		
		add_user_voice(0,USER_BUDGET,budget_incr,
			CALLS_PER_SECOND,duration,nodechooser);
	}

	public float run() {
		init();

		// create links
		lf =  FixedDelayLink.randomdelay_factory(s,10000000,10000,20000,0);
		lmf = FairLinkManager.factory(s);
		t.add_duplex_link(0,1,lf,lmf);
		t.add_duplex_link(0,4,lf,lmf);
		t.add_duplex_link(0,9,lf,lmf);
		t.add_duplex_link(1,2,lf,lmf);
		t.add_duplex_link(1,6,lf,lmf);
		t.add_duplex_link(2,3,lf,lmf);
		t.add_duplex_link(2,4,lf,lmf);
		t.add_duplex_link(3,7,lf,lmf);
		t.add_duplex_link(4,5,lf,lmf);
		t.add_duplex_link(4,8,lf,lmf);
		t.add_duplex_link(5,6,lf,lmf);
		t.add_duplex_link(6,7,lf,lmf);
		t.add_duplex_link(7,9,lf,lmf);
		t.add_duplex_link(7,11,lf,lmf);
		t.add_duplex_link(8,9,lf,lmf);
		t.add_duplex_link(9,10,lf,lmf);
		t.add_duplex_link(10,11,lf,lmf);

		// create nam log
		//t.make_nam_log("testslatr.nam");

		periodic_debug();

		s.run(1*1000*1000); // 1 seconds

		add_users();

		s.run(5*1000*1000);

		//print_routing_tables_dv();
		print_routing_tables_slatr(System.out);

		t.flush_logs();

		User user = (User) t.node(0).get_service(USER_PORT);

		// System.out.println("max="+user.get_max_utility());

		return user.get_utility()/user.get_max_utility();
	}

	void print_routing_tables_slatr(PrintStream ps) {
		for(int i=0; i<t.nodes(); i++) {
			ps.println();
			SLATRouter r = (SLATRouter) t.node(i).get_service(SLATROUTER_PORT);
			r.debug_table(ps);
		}
	}

	class PeriodicDebugEvent implements Event {
		public void trigger() { periodic_debug(); }
	}

	void periodic_debug() {
		try {
			FileOutputStream fos = new FileOutputStream("pdebug/pdebug-"+s.now());
			PrintStream ps = new PrintStream(fos);
			print_routing_tables_slatr(ps);
			s.schedule(PERIODIC_DEBUG_PERIOD,new PeriodicDebugEvent());
		}
		catch(IOException e) {}
	}

}
