package flowsim.slatr;

import flowsim.core.*;

/**
 * Generic/common User implementation.
 */
abstract public class UserImpl implements User, Service
{
	static final boolean DEBUG=false;
	static final int BUDGET_INCREASE_TIME = 100000;

	protected final Simulator s;
	protected final Node node;
	protected final int port;
	protected long budget;
	protected long budget_incr;
	protected float max_utility;
	protected float utility;

	class BudgetIncreaseEvent implements Event {
		public void trigger() {
			budget += budget_incr*1000000/BUDGET_INCREASE_TIME;
			s.schedule(BUDGET_INCREASE_TIME, this);
		}
	}

	public Node get_node() {
		return node;
	}

	public int get_port() {
		return port;
	}

	public long get_budget() {
		// Don't use it. Privacy! :-)
		return budget;
	}

	/** returns by how much the budget is increased per second. */
	public long get_budget_incr() {
		return budget_incr;
	}

	public final float get_max_utility() {
		return max_utility;
	}

	public final float get_utility() {
		return utility;
	}

	public void add_to_budget(long money) {
		// Wanna make a donation?
		budget+=money;
	}

	public void remove_from_budget(long money) {
		budget-=money;
	}

	public String toString() {
		return "SLATUser("+get_node()+","+get_port()+")";
	}

	public void print_stats() {
		System.out.println(this+": budget="+budget+", budget_incr="+budget_incr+
		", max_utility="+get_max_utility()+", utility="+get_utility());
	}

	/**
	 * @param s simulator object.
	 * @param node node where the user resides.
	 * @param port port where the user resides.
	 * @param budget initial budget.
	 * @param budget_incr budget increase pro second.
	 * @see User
	 */
	public UserImpl(Simulator s, Node node, int port, long budget, long budget_incr)
	{
		this.s    = s;
		this.node = node;
		this.port = port;
		this.budget=budget;
		this.budget_incr = (long)(budget_incr);
		this.max_utility = this.utility = 0;

		if(budget_incr>0) s.schedule(BUDGET_INCREASE_TIME, new BudgetIncreaseEvent());

		/* add to node */
		node.set_service(this, port);
	}
}
