package flowsim.slatr;

import flowsim.core.*;
import flowsim.util.*;

import java.util.Random;

public class UserVoice extends UserImpl implements FlowMonitor
{
	static final int BANDWIDTH  = 16*1000; // 16 kbps
	static final int DELAY_GOOD = 250*1000; // 50 ms (should be)

	protected final Topology t;
	protected final Random r;
	protected final Distribution arrival;
	protected final Distribution duration;
	protected final NodeChooser to_chooser;
	protected final int calls_per_second;
	protected final int flow_id;
	protected final int router_port;

	class MakeCallEvent implements Event {
		public void trigger() {
			make_call();
		}
	}

	final class StopFlowEvent implements Event {
		final Flow flow;
		public void trigger() {
			if(!flow.is_running()) utility--;
			flow.stop();
		}
		StopFlowEvent(final Flow flow) { this.flow = flow; }
	}

	public void flow_started(Flow f) {
		if(DEBUG) s.debug(this,"Flow "+f+" started");
	}

	public void flow_stopped(Flow f) {
		if(DEBUG) s.debug(this,"Flow "+f+" stopped");
	}

	public void flow_blocked(Flow f) {
		if(DEBUG) s.debug(this,"Flow "+f+" blocked");
		//s.error(this,"Flow "+f+" was blocked!!");
	}

	public void flow_changed_bw(Flow f, int was_bw) {
		s.error(this,"Flow "+f+" changed bandwidth!!");
	}

	public void flow_changed_desired_bw(Flow f, int was_bw) {}
	public void flow_changed_available_bw(Flow f, int was_bw) {}

	public void start_flow(Node dest, long duration) {
		RoutedFlow f = new UnicastFlowImpl(s, get_node(), dest, BANDWIDTH, flow_id,
			0, router_port);
		f.attach_monitor(this);
		
		if(DEBUG) s.debug(this,"Starting flow "+f+" (stop="+duration+")");

		max_utility++;
		utility++;

		f.start();

		s.schedule(duration, new StopFlowEvent(f));
	}
	
	void make_call() {
		// destination
		Node to;
		do {
			to = to_chooser.get();
		} while(get_node() == to);

		// duration
		long d = (long) duration.rand();

		start_flow(to, d);

		s.schedule((long) arrival.rand(),new MakeCallEvent());
	}

	/**
	 * @param node node where the user resides
	 * @param port port where the user resides
	 * @param budget_incr budget increase pro second
	 * @param server_port port of (unique) SLASeller for this user
	 * @param calls_per_second number of calls per second which should be generated (randomly)
	 * @param mean_duration average duration of calls in microseconds
	 * @param r Random object used to make random call distribution
	 */
	public UserVoice(Simulator s, Topology t, Node node, int port,
		int router_port, int calls_per_second, int mean_duration,
		Random r, NodeChooser to_chooser, int flow_id)
	{
		super(s,node,port,0,0);
		this.t = t;
		this.router_port = router_port;
		this.calls_per_second = calls_per_second;
		this.r = r;
		this.to_chooser = to_chooser;
		this.flow_id = flow_id;
		this.arrival = new ExponentialDistribution((double)calls_per_second/1000000,r);
		this.duration = new ExponentialDistribution(1.0/mean_duration,r);

		s.schedule((long) arrival.rand(),new MakeCallEvent());

		if(DEBUG) s.debug(this,"UserVoice instantiated");
	}
}
